@@ -1008,6 +1008,9 @@ descriptive_stat_tool (WorkbookControl *wbc, Sheet *current_sheet,
 	        free_data_set (&data_sets[i]);
 	g_free (data_sets);
 
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (current_sheet);
+
 	return 0;
 }
 
@@ -1118,6 +1121,8 @@ int sampling_tool (WorkbookControl *wbc, Sheet *sheet, Range *input_range,
 	}
 
 	free_data_set (&data_set);
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
 
 	return 0;
 }
@@ -1217,6 +1222,9 @@ int ztest_tool (WorkbookControl *wbc, Sheet *sheet, Range *input_range1,
 	free_data_set (&set_one);
 	free_data_set (&set_two);
 
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
         return 0;
 }
 
@@ -1433,13 +1441,13 @@ ttest_eq_var_tool (WorkbookControl *wbc, Sheet *sheet, Range *input_range1,
 	var1 = (set_one.sqrsum - set_one.sum2 / set_one.n) / (set_one.n - 1);
 	var2 = (set_two.sqrsum - set_two.sum2 / set_two.n) / (set_two.n - 1);
 
-	var = (set_one.sqrsum + set_two.sqrsum - (set_one.sum + set_two.sum) *
-	       (set_one.sum + set_two.sum)/ (set_one.n + set_two.n)) /
-		(set_one.n + set_two.n - 1);  /* TODO: Correct??? */
-
 	df = set_one.n + set_two.n - 2;
+
+	var = ((set_one.sqrsum - set_one.sum2 / set_one.n) +
+	       (set_two.sqrsum - set_two.sum2 / set_two.n)) / df;
+
 	t = fabs (mean1 - mean2 - mean_diff) /
-		sqrt (var1 / set_one.n + var2 / set_two.n);
+		sqrt (var / set_one.n + var / set_two.n);
 	p = 1.0 - pt (t, df);
 
 	/* Mean */
@@ -1872,6 +1880,9 @@ int random_tool (WorkbookControl *wbc, Sheet *sheet, int vars, int count,
 		break;
 	}
 
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
 	return 0;
 }
 
@@ -2155,6 +2166,10 @@ these values can be tiny.*/
 		g_free (xss[i]);
 	g_free (xss);
 	g_free (res);
+
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
 	return 0;
 }
 
@@ -2216,6 +2231,9 @@ int average_tool (WorkbookControl *wbc, Sheet *sheet, Range *range, int interval
 
 	free_data_set (&data_set);
 
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
 	return 0;
 }
 
@@ -2362,6 +2380,9 @@ int ranking_tool (WorkbookControl *wbc, Sheet *sheet, Range *input_range,
 	        free_data_set (&data_sets[i]);
 	g_free (data_sets);
 
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
 	return 0;
 }
 
@@ -2529,6 +2550,9 @@ anova_single_factor_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 	        free_data_set (&data_sets[i]);
 	g_free (data_sets);
 
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
         return 0;
 }
 
@@ -2685,6 +2709,9 @@ anova_two_factor_without_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *rang
 	set_cell_float (dao, 6, 8 + rows + cols, f1_c);
 	set_cell_float (dao, 6, 9 + rows + cols, f2_c);
 
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
 	return 0;
 }
 
@@ -2701,18 +2728,27 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 			      int rows_per_sample, gnum_float alpha,
 			      data_analysis_output_t *dao)
 {
-	int        cols, rows, i, j, k, n;
-	int        count, gr_count;
+	int        cols, rows, i, j, k, n, a, b;
+	int        count, gr_count, tot_count;
 	gnum_float    sum, sqrsum, x, v;
 	gnum_float    gr_sum, gr_sqrsum;
+	gnum_float    tot_sum, tot_sqrsum;
 	int        *col_count;
 	gnum_float    *col_sum, *col_sqrsum;
+	gnum_float    col_sum_sqrsum, gr_sum_sqrsum, sample_sum_sqrsum;
+	gnum_float    df_col, df_gr, df_inter, df_within;
+	gnum_float    ss_col, ss_gr, ss_inter, ss_within, ss_tot;
+	gnum_float    ms_col, ms_gr, ms_inter, ms_within;
+	gnum_float    f_col, f_gr, f_inter;
+	gnum_float    p_col, p_gr, p_inter;
 	Cell       *cell;
 
 	cols = range->end.col - range->start.col + 1;
 	rows = range->end.row - range->start.row + 1;
 
 	/* Check that correct number of rows per sample */
+	if (rows_per_sample == 0)
+	        return 1;
 	if ((rows - 1) % rows_per_sample != 0)
 	        return 1;
 
@@ -2752,6 +2788,14 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 	set_italic (dao, 1, 2, i - 1, 2);
 	set_cell (dao, i, 2, _("Total"));
 
+	tot_count = 0;
+	tot_sum = 0;
+	tot_sqrsum = 0;
+
+	sample_sum_sqrsum = 0;
+
+	gr_sum_sqrsum = 0;
+
 	for (i = n = 0; i < rows - 1; i += rows_per_sample) {
 	       cell = sheet_cell_get (sheet, range->start.col,
 				      range->start.row + i + 1);
@@ -2791,6 +2835,12 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 		      gr_sum += sum;
 		      gr_sqrsum += sqrsum;
 
+  		      sample_sum_sqrsum += sum * sum;
+
+		      tot_count += count;
+		      tot_sum += sum;
+		      tot_sqrsum += sqrsum;
+
 		      set_cell_int   (dao, j, n * 6 + 4, count);
 		      set_cell_float (dao, j, n * 6 + 5, sum);
 		      set_cell_float (dao, j, n * 6 + 6, sum / count);
@@ -2802,6 +2852,8 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 	       set_cell_float (dao, j, n * 6 + 6, gr_sum / gr_count);
 	       set_cell_float (dao, j, n * 6 + 7, v);
 
+	       gr_sum_sqrsum += (gr_sum * gr_sum);
+
 	       ++n;
 	}
 
@@ -2812,6 +2864,8 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 	set_cell (dao, 0, n * 6 + 6, _("Average"));
 	set_cell (dao, 0, n * 6 + 7, _("Variance"));
 
+	col_sum_sqrsum = 0;
+
 	for (i = 1; i < cols; i++) {
 	        v = (col_sqrsum[i - 1] - (col_sum[i - 1] * col_sum[i - 1]) /
 		     col_count[i - 1]) / (col_count[i - 1] - 1);
@@ -2819,8 +2873,35 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 	        set_cell_float (dao, i, n * 6 + 5, col_sum[i - 1]);
 	        set_cell_float (dao, i, n * 6 + 6, col_sum[i - 1] / col_count[i - 1]);
 	        set_cell_float (dao, i, n * 6 + 7, v);
+		col_sum_sqrsum +=  (col_sum[i - 1] * col_sum[i - 1]);
 	}
 
+	a = (rows - 1)/rows_per_sample;
+	b = cols -1;
+	ss_gr = (gr_sum_sqrsum - (tot_sum*tot_sum)/a) / (b * rows_per_sample) ;
+	ss_col = (col_sum_sqrsum - (tot_sum*tot_sum)/b) / (a * rows_per_sample) ;
+	ss_within = tot_sqrsum - sample_sum_sqrsum / rows_per_sample;
+	ss_tot = tot_sqrsum - (tot_sum*tot_sum) / (a * b * rows_per_sample);
+	ss_inter = ss_tot - ss_within - ss_col - ss_gr;
+
+	df_gr = a - 1;
+	df_col = b - 1;
+	df_inter = (a -1) * (b - 1);
+	df_within = a * b * (rows_per_sample - 1);
+
+	ms_gr = ss_gr / df_gr;
+	ms_col = ss_col / df_col;
+	ms_inter = ss_inter / df_inter;
+	ms_within = ss_within / df_within;
+
+	f_gr = ms_gr / ms_within;
+	f_col = ms_col / ms_within;
+	f_inter = ms_inter / ms_within;
+
+	p_gr = 1.0 - pf(f_gr,df_gr,df_within);
+	p_col = 1.0 - pf(f_col,df_col,df_within);
+	p_inter = 1.0 - pf(f_inter,df_inter,df_within);
+
 	set_cell (dao, 0, n * 6 + 10, _("ANOVA"));
 	set_cell (dao, 0, n * 6 + 11, _("Source of Variation"));
 	set_cell (dao, 0, n * 6 + 12, _("Sample"));
@@ -2830,20 +2911,57 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
 	set_cell (dao, 0, n * 6 + 17, _("Total"));
 
 	set_cell (dao, 1, n * 6 + 11, _("SS"));
+	set_cell_float (dao, 1, n * 6 + 12, ss_gr);
+	set_cell_float (dao, 1, n * 6 + 13, ss_col);
+	set_cell_float (dao, 1, n * 6 + 14, ss_inter);
+	set_cell_float (dao, 1, n * 6 + 15, ss_within);
+	set_cell_float (dao, 1, n * 6 + 17, ss_tot);
+
 	set_cell (dao, 2, n * 6 + 11, _("df"));
+	set_cell_int   (dao, 2, n * 6 + 12, df_gr);
+	set_cell_int   (dao, 2, n * 6 + 13, df_col);
+	set_cell_int   (dao, 2, n * 6 + 14, df_inter);
+	set_cell_int   (dao, 2, n * 6 + 15, df_within);
+	set_cell_int   (dao, 2, n * 6 + 17, tot_count-1);
+
+	/* Note: a * b * rows_per_sample, tot_count and df_gr+df_col+df_inter+df_within should all be the same*/
+
 	set_cell (dao, 3, n * 6 + 11, _("MS"));
+	set_cell_float (dao, 3, n * 6 + 12, ms_gr);
+	set_cell_float (dao, 3, n * 6 + 13, ms_col);
+	set_cell_float (dao, 3, n * 6 + 14, ms_inter);
+	set_cell_float (dao, 3, n * 6 + 15, ms_within);
+
+
 	set_cell (dao, 4, n * 6 + 11, _("F"));
+	set_cell_float (dao, 4, n * 6 + 12, f_gr);
+	set_cell_float (dao, 4, n * 6 + 13, f_col);
+	set_cell_float (dao, 4, n * 6 + 14, f_inter);
+
+
 	set_cell (dao, 5, n * 6 + 11, _("P-value"));
+	set_cell_float (dao, 5, n * 6 + 12, p_gr);
+	set_cell_float (dao, 5, n * 6 + 13, p_col);
+	set_cell_float (dao, 5, n * 6 + 14, p_inter);
+
 	set_cell (dao, 6, n * 6 + 11, _("F crit"));
+	set_cell_float (dao, 6, n * 6 + 12, qf(alpha,df_gr,df_within));
+	set_cell_float (dao, 6, n * 6 + 13, qf(alpha,df_col,df_within));
+	set_cell_float (dao, 6, n * 6 + 14, qf(alpha,df_inter,df_within));
 
 	set_italic (dao, 0, n * 6 + 11, 6, n * 6 + 11);
 
-	/* FIXME: Implement the rest of the ANOVA computations. */
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
+	g_free(col_count);
+	g_free(col_sum);
+	g_free(col_sqrsum);
 
 	return 0;
 }
 
-
+
 /************* Histogram Tool *********************************************
  *
  * The results are given in a table which can be printed out in a new
@@ -2851,14 +2969,15 @@ anova_two_factor_with_r_tool (WorkbookControl *wbc, Sheet *sheet, Range *range,
  *
  **/
 
-int histogram_tool (WorkbookControl *wbc, Sheet *sheet, Range *range, Range *bin_range,
-		    gboolean labels, gboolean sorted, gboolean percentage,
-		    gboolean chart, data_analysis_output_t *dao)
+int
+histogram_tool (WorkbookControl *wbc, Sheet *sheet, Range *range, Range *bin_range,
+		gboolean labels, gboolean sorted, gboolean percentage,
+		gboolean chart, data_analysis_output_t *dao)
 {
         data_set_t bin_set, set;
 	GSList     *list;
 	int        i, j, cols, rows, cum_sum;
-	gnum_float    *intval;
+	gnum_float *intval;
 	int        *count;
 
 	cols = bin_range->end.col - bin_range->start.col + 1;
@@ -2928,5 +3047,9 @@ int histogram_tool (WorkbookControl *wbc, Sheet *sheet, Range *range, Range *bin
 
 	free_data_set (&set);
 	free_data_set (&bin_set);
+
+	sheet_set_dirty (dao->sheet, TRUE);
+	sheet_update (sheet);
+
 	return 0;
 }
