@@ -244,13 +244,11 @@ colrow_save_sizes (Sheet *sheet, gboolean const is_cols, int first, int last)
 	float              size, run_size = 0.;
 	int                run_length = 0;
 
-	g_return_val_if_fail (sheet != NULL, NULL);
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 	g_return_val_if_fail (first <= last, NULL);
 
 	for (i = first; i <= last; ++i) {
-		ColRowInfo *info = is_cols
-		    ? sheet_col_get_info (sheet, i)
-		    : sheet_row_get_info (sheet, i);
+		ColRowInfo *info = sheet_colrow_get_info (sheet, i, is_cols);
 
 		g_return_val_if_fail (info != NULL, NULL); /* be anal, and leak */
 		
@@ -395,7 +393,7 @@ colrow_restore_sizes (Sheet *sheet, gboolean const is_cols,
 	int i, offset = first;
 
 	g_return_if_fail (sizes != NULL);
-	g_return_if_fail (sheet != NULL);
+	g_return_if_fail (IS_SHEET (sheet));
 	g_return_if_fail (first <= last);
 
 	for (l = sizes; l != NULL; l = l->next) {
@@ -818,7 +816,8 @@ colrow_set_visibility (Sheet *sheet, gboolean const is_cols,
 {
 	int i, prev_outline = 0;
 	gboolean prev_changed = FALSE;
-	g_return_if_fail (sheet != NULL);
+
+	g_return_if_fail (IS_SHEET (sheet));
 
 	for (i = first; i <= last ; ++i) {
 		ColRowInfo * const cri = is_cols
@@ -833,6 +832,14 @@ colrow_set_visibility (Sheet *sheet, gboolean const is_cols,
 			cri->visible = visible;
 			prev_outline = cri->outline_level;
 			sheet->priv->recompute_visibility = TRUE;
+
+			if (is_cols) {
+				if (sheet->priv->reposition_objects.col > i)
+					sheet->priv->reposition_objects.col = i;
+			} else {
+				if (sheet->priv->reposition_objects.row > i)
+					sheet->priv->reposition_objects.row = i;
+			}
 		}
 	}
 	if (prev_changed &&
