@@ -81,6 +81,9 @@ void		   colrow_restore_sizes_group	(Sheet *sheet, gboolean const is_cols,
 						 int old_size);
 
 /* Support for Col/Row visibility */
+int              colrow_set_outline             (ColRowInfo *cri, gboolean is_cols,
+						 int outline_level, gboolean relative,
+						 gboolean is_collapsed);
 void		 colrow_adjust_outline_dir	(ColRowCollection *colrows,
 						 gboolean pre_or_post);
 int		 colrow_find_outline_bound	(Sheet const *sheet, gboolean is_cols,
