@@ -2,7 +2,11 @@
 #define GNUMERIC_FILE_H
 
 #include <gtk/gtktypeutils.h>
+#include "io-context.h"
 #include "gnumeric.h"
+#ifdef ENABLE_BONOBO
+#include <bonobo/bonobo-stream.h>
+#endif
 
 /*
  * File format levels. They are ordered. When we save a file, we
@@ -95,6 +99,13 @@ typedef void (*GnumFileSaverSaveFunc) (GnumFileSaver const *fs,
                                        IOContext *io_context,
                                        WorkbookView *wbv,
                                        const gchar *file_name);
+#ifdef ENABLE_BONOBO
+typedef void (*GnumFileSaverSaveToStreamFunc) (GnumFileSaver const  *fs,
+					       IOContext *io_context,
+					       WorkbookView *wbv,
+					       BonoboStream *stream,
+					       CORBA_Environment *ev);
+#endif
 
 GtkType gnum_file_saver_get_type (void);
 
@@ -109,10 +120,18 @@ FileSaveScope gnum_file_saver_get_save_scope (GnumFileSaver *fs);
 
 void         gnum_file_saver_save (GnumFileSaver const *fs, IOContext *io_context,
                                    WorkbookView *wbv, const gchar *file_name);
+#ifdef ENABLE_BONOBO
+void         gnum_file_saver_save_to_stream (GnumFileSaver const *fs, 
+                                             IOContext *io_context,
+                                             WorkbookView *wbv, 
+                                             BonoboStream *stream,
+                                             CORBA_Environment *ev);
+#endif
 gchar       *gnum_file_saver_fix_file_name (GnumFileSaver const *fs,
                                             const gchar *file_name);
 const gchar *gnum_file_saver_get_id (GnumFileSaver const *fs);
 const gchar *gnum_file_saver_get_extension (GnumFileSaver const *fs);
+const gchar *gnum_file_saver_get_mime_type (GnumFileSaver const *fs);
 const gchar *gnum_file_saver_get_description (GnumFileSaver const *fs);
 FileFormatLevel gnum_file_saver_get_format_level (GnumFileSaver const *fs);
 
@@ -129,6 +148,7 @@ void register_file_saver_as_default (GnumFileSaver *fs, gint priority);
 void unregister_file_saver (GnumFileSaver *fs);
 
 GnumFileSaver *get_default_file_saver (void);
+GnumFileSaver *get_file_saver_for_mime_type (const gchar *mime_type);
 GnumFileOpener *get_file_opener_by_id (const gchar *id);
 GnumFileSaver *get_file_saver_by_id (const gchar *id);
 
