@@ -12,8 +12,10 @@
 #include "item-cursor.h"
 #include "item-edit.h"
 #include "item-grid.h"
-#include "sheet-control-gui.h"
+#include "sheet-control-gui-priv.h"
 #include "gnumeric-util.h"
+#include "mstyle.h"
+#include "style-border.h"
 #include "style-color.h"
 #include "selection.h"
 #include "parse-util.h"
@@ -30,6 +32,7 @@
 #  include "sheet-object-container.h"
 #endif
 #include <gal/widgets/e-cursors.h>
+#include <gal/util/e-util.h>
 
 static GnomeCanvasClass *gsheet_parent_class;
 
@@ -45,15 +48,42 @@ gnumeric_sheet_destroy (GtkObject *object)
 		(*GTK_OBJECT_CLASS (gsheet_parent_class)->destroy)(object);
 }
 
+/*
+ * Adds borders to all the selected regions on the sheet.
+ * FIXME: This is a little more simplistic then it should be, it always
+ * removes and/or overwrites any borders. What we should do is
+ * 1) When adding -> don't add a border if the border is thicker than 'THIN'
+ * 2) When removing -> don't remove unless the border is 'THIN'
+ */
+static void
+borders_mutate (WorkbookControlGUI *wbcg, Sheet *sheet, gboolean add)
+{
+	StyleBorder *borders[STYLE_BORDER_EDGE_MAX];
+	int i;
+
+	for (i = STYLE_BORDER_TOP; i < STYLE_BORDER_EDGE_MAX; ++i)
+		if (i <= STYLE_BORDER_RIGHT)
+			borders[i] = style_border_fetch (
+				add ? STYLE_BORDER_THIN : STYLE_BORDER_NONE,
+				style_color_black (), style_border_get_orientation (i));
+		else
+			borders[i] = NULL;
+								  
+	cmd_format (WORKBOOK_CONTROL (wbcg), sheet, NULL, borders,
+		    add ? _("Add Borders") : _("Remove borders"));
+}
+
 /*
  * key press event handler for the gnumeric sheet for the sheet mode
  */
 static gint
 gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 {
-	Sheet *sheet = gsheet->scg->sheet;
+	SheetControl *sc = (SheetControl *) gsheet->scg;
+	Sheet *sheet = sc->sheet;
 	WorkbookControlGUI *wbcg = gsheet->scg->wbcg;
 	gboolean const jump_to_bounds = event->state & GDK_CONTROL_MASK;
+	int state = gnumeric_filter_modifiers (event->state);
 	void (*movefn) (SheetControlGUI *, int n,
 			gboolean jump, gboolean horiz);
 
@@ -82,25 +112,99 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 			: scg_cursor_move;
 	}
 
+	if ((event->state & GDK_CONTROL_MASK) && (event->state & GDK_SHIFT_MASK)) {
+		char *fmt = NULL;
+		char *desc = NULL;
+		
+		switch (event->keyval) {
+		case GDK_asciitilde :
+			fmt = "0";
+			desc = _("Format as Number");
+			break;
+		case GDK_dollar :
+			fmt = "$0.00_);($0.00)";
+			desc = _("Format as Currency");
+			break;
+		case GDK_percent :
+			fmt = "0%";
+			desc = _("Format as Percentage");
+			break;
+		case GDK_asciicircum :
+			fmt = "0.00E+00";
+			desc = _("Format as Scientific");
+			break;
+		case GDK_numbersign :
+			fmt = "d-mmm-yy";
+			desc = _("Format as Date");
+			break;
+		case GDK_at :
+			fmt = "h:mm AM/PM";
+			desc = _("Format as Time");
+			break;
+		case GDK_exclam :
+			fmt = "#,##0.00";
+			desc = _("Format as alternative Number"); /* FIXME: Better descriptor */
+			break;
+		case GDK_ampersand :
+			borders_mutate (wbcg, sheet, TRUE);
+			return TRUE;
+		case GDK_underscore :
+			borders_mutate (wbcg, sheet, FALSE);
+			return TRUE;
+		}
+
+		if (fmt) {
+			MStyle *mstyle = mstyle_new ();
+
+			mstyle_set_format_text (mstyle, fmt);
+			cmd_format (WORKBOOK_CONTROL (wbcg), sheet, mstyle, NULL, desc);
+			return TRUE;
+		}
+	}
+	
 	switch (event->keyval) {
 	case GDK_KP_Left:
 	case GDK_Left:
-		(*movefn) (gsheet->scg, -1, jump_to_bounds, TRUE);
+		if (event->state & GDK_MOD5_MASK) /* Scroll Lock */
+			scg_set_left_col (gsheet->scg,
+					  gsheet->col.first > 0
+					  ? gsheet->col.first - 1
+					  : 0);
+		else
+			(*movefn) (gsheet->scg, -1, jump_to_bounds, TRUE);
 		break;
 
 	case GDK_KP_Right:
 	case GDK_Right:
-		(*movefn) (gsheet->scg, 1, jump_to_bounds, TRUE);
+		if (event->state & GDK_MOD5_MASK) /* Scroll Lock */
+			scg_set_left_col (gsheet->scg,
+					  gsheet->col.first < SHEET_MAX_COLS - 1
+					  ? gsheet->col.first + 1
+					  : SHEET_MAX_COLS - 1);
+		else
+			(*movefn) (gsheet->scg, 1, jump_to_bounds, TRUE);
 		break;
 
 	case GDK_KP_Up:
 	case GDK_Up:
-		(*movefn) (gsheet->scg, -1, jump_to_bounds, FALSE);
+		if (event->state & GDK_MOD5_MASK) /* Scroll Lock */
+			scg_set_top_row (gsheet->scg,
+					 gsheet->row.first > 0
+					 ? gsheet->row.first - 1
+					 : 0);
+		else
+			(*movefn) (gsheet->scg, -1, jump_to_bounds, FALSE);
 		break;
 
 	case GDK_KP_Down:
 	case GDK_Down:
-		(*movefn) (gsheet->scg, 1, jump_to_bounds, FALSE);
+		if (event->state & GDK_MOD5_MASK) /* Scroll Lock */
+			scg_set_top_row (gsheet->scg,
+					 gsheet->row.first < SHEET_MAX_ROWS - 1
+					 ? gsheet->row.first + 1
+					 : SHEET_MAX_ROWS - 1);
+		else
+			(*movefn) (gsheet->scg, 1, jump_to_bounds, FALSE);
 		break;
 
 	case GDK_KP_Page_Up:
@@ -133,12 +237,37 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 
 	case GDK_KP_Home:
 	case GDK_Home:
-		/* do the ctrl-home jump to A1 in 2 steps */
-		(*movefn)(gsheet->scg, -SHEET_MAX_COLS, FALSE, TRUE);
-		if ((event->state & GDK_CONTROL_MASK))
-			(*movefn)(gsheet->scg, -SHEET_MAX_ROWS, FALSE, FALSE);
+		if (event->state & GDK_MOD5_MASK) { /* Scroll Lock */
+			scg_set_left_col (gsheet->scg, sheet->edit_pos.col);
+			scg_set_top_row (gsheet->scg, sheet->edit_pos.row);
+		} else {
+			/* do the ctrl-home jump to A1 in 2 steps */
+			(*movefn)(gsheet->scg, -SHEET_MAX_COLS, FALSE, TRUE);
+			if ((event->state & GDK_CONTROL_MASK))
+				(*movefn)(gsheet->scg, -SHEET_MAX_ROWS, FALSE, FALSE);
+		}
 		break;
 
+	case GDK_KP_End:
+	case GDK_End:
+		if (event->state & GDK_MOD5_MASK) { /* Scroll Lock */
+			int new_col = sheet->edit_pos.col - (gsheet->col.last_full - gsheet->col.first);
+			int new_row = sheet->edit_pos.row - (gsheet->row.last_full - gsheet->row.first);
+			
+			if (new_col > 0)
+				scg_set_left_col (gsheet->scg, new_col);
+			if (new_row > 0)
+				scg_set_top_row (gsheet->scg, new_row);
+		} else {
+			Range r = sheet_get_extent (sheet, FALSE);
+		
+			/* do the ctrl-end jump to the extent in 2 steps */
+			(*movefn)(gsheet->scg, r.end.col - sheet->edit_pos.col, FALSE, TRUE);
+			if ((event->state & GDK_CONTROL_MASK))
+				(*movefn)(gsheet->scg, r.end.row - sheet->edit_pos.row, FALSE, FALSE);
+		}
+		break;
+	
 	case GDK_KP_Delete:
 	case GDK_Delete:
 		cmd_clear_selection (WORKBOOK_CONTROL (wbcg), sheet, CLEAR_VALUES);
@@ -150,9 +279,9 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	 */
 	case GDK_KP_Enter:
 	case GDK_Return:
-		if (wbcg_rangesel_possible (wbcg) &&
-		    (event->state == GDK_CONTROL_MASK ||
-		     event->state == (GDK_CONTROL_MASK|GDK_SHIFT_MASK) ||
+		if (wbcg->editing &&
+		    (state == GDK_CONTROL_MASK ||
+		     state == (GDK_CONTROL_MASK|GDK_SHIFT_MASK) ||
 		     event->state == GDK_MOD1_MASK))
 			/* Forward the keystroke to the input line */
 			return gtk_widget_event (
@@ -183,7 +312,7 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 		wbcg_edit_finish (wbcg, FALSE);
 		application_clipboard_unant ();
 		break;
-
+		
 	case GDK_F4:
 		if (gsheet->sel_cursor)
 			wbcg_edit_toggle_absolute (wbcg);
@@ -193,6 +322,14 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 		wbcg_edit_start (wbcg, FALSE, FALSE);
 		/* fall down */
 
+	case GDK_BackSpace:
+		/* Re-center the view on the active cell */
+		if (!wbcg->editing && (event->state & GDK_CONTROL_MASK) != 0) {
+			scg_make_cell_visible (gsheet->scg, sheet->edit_pos.col,
+					       sheet->edit_pos.row, FALSE, TRUE);
+			break;
+		}
+
 	default:
 		if (!wbcg->editing) {
 			if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0)
@@ -226,7 +363,7 @@ gnumeric_sheet_key_mode_object (GnumericSheet *gsheet, GdkEventKey *event)
 
 	switch (event->keyval) {
 	case GDK_Escape:
-		scg_mode_edit (scg);
+		scg_mode_edit ((SheetControl *) scg);
 		application_clipboard_unant ();
 		break;
 
@@ -257,7 +394,7 @@ static gint
 gnumeric_sheet_key_release (GtkWidget *widget, GdkEventKey *event)
 {
 	GnumericSheet *gsheet = GNUMERIC_SHEET (widget);
-	SheetControlGUI *scg = gsheet->scg;
+	SheetControl *sc = (SheetControl *) gsheet->scg;
 
 	/*
 	 * The status_region normally displays the current edit_pos
@@ -266,10 +403,10 @@ gnumeric_sheet_key_release (GtkWidget *widget, GdkEventKey *event)
 	 * is released, or the mouse button is release we need to reset
 	 * to displaying the edit pos.
 	 */
-	if (scg->current_object == NULL &&
+	if (gsheet->scg->current_object == NULL &&
 	    (event->keyval == GDK_Shift_L || event->keyval == GDK_Shift_R))
 		wb_view_selection_desc (wb_control_view (
-			WORKBOOK_CONTROL (gsheet->scg->wbcg)), TRUE, NULL);
+			sc->wbc), TRUE, NULL);
 
 	return (*GTK_WIDGET_CLASS (gsheet_parent_class)->key_release_event)(widget, event);
 }
@@ -277,7 +414,10 @@ gnumeric_sheet_key_release (GtkWidget *widget, GdkEventKey *event)
 static gint
 gnumeric_sheet_button_release (GtkWidget *widget, GdkEventButton *button)
 {
-	GnumericSheet *gsheet = GNUMERIC_SHEET (widget);
+	GnumericSheet *current = GNUMERIC_SHEET (widget);
+
+	/* scroll always operates on pane 0 */
+	GnumericSheet *gsheet = scg_pane (current->scg, 0);
 
 	if (button->button != 4 && button->button != 5)
 		return (*GTK_WIDGET_CLASS (gsheet_parent_class)->button_release_event)(widget, button);
@@ -285,23 +425,27 @@ gnumeric_sheet_button_release (GtkWidget *widget, GdkEventButton *button)
 	/* Roll Up or Left */
 	/* Roll Down or Right */
 	if ((button->state & GDK_MOD1_MASK)) {
-		int col = gsheet->col.last_full - gsheet->col.first;
+		int col = (gsheet->col.last_full - gsheet->col.first) / 4;
+		if (col < 1)
+			col = 1;
 		if (button->button == 4)
-			col = MAX (gsheet->col.first - col, 0);
-		else if (gsheet->col.last_full < SHEET_MAX_COLS-1)
-			col = gsheet->col.last_full;
+			col = gsheet->col.first - col;
 		else
-			return FALSE;
-		gnumeric_sheet_set_left_col (gsheet, col);
+			col = gsheet->col.first + col;
+
+		if (0 <= col && col <= SHEET_MAX_COLS-1)
+			scg_set_left_col (gsheet->scg, col);
 	} else {
-		int row = gsheet->row.last_full - gsheet->row.first;
+		int row = (gsheet->row.last_full - gsheet->row.first) / 4;
+		if (row < 1)
+			row = 1;
 		if (button->button == 4)
-			row = MAX (gsheet->row.first - row, 0);
-		else if (gsheet->row.last_full < SHEET_MAX_ROWS-1)
-			row = gsheet->row.last_full;
+			row = gsheet->row.first - row;
 		else
-			return FALSE;
-		gnumeric_sheet_set_top_row (gsheet, row);
+			row = gsheet->row.first + row;
+
+		if (0 <= row && row <= SHEET_MAX_ROWS-1)
+			scg_set_top_row (gsheet->scg, row);
 	}
 	return TRUE;
 }
@@ -372,21 +516,19 @@ gnumeric_sheet_filenames_dropped (GtkWidget        *widget,
 				  GnumericSheet    *gsheet)
 {
 	GList *names, *tmp_list;
-	WorkbookControl *wbc = WORKBOOK_CONTROL (gsheet->scg->wbcg);
+	SheetControl *sc = (SheetControl *) gsheet->scg;
+	WorkbookControl *wbc = sc->wbc;
 
 	names = gnome_uri_list_extract_filenames ((char *)selection_data->data);
 
 	for (tmp_list = names; tmp_list != NULL; tmp_list = tmp_list->next) {
 		gchar *file_name = tmp_list->data;
-		WorkbookView *new_wb = workbook_try_read (wbc, file_name);
 
-		if (new_wb != NULL) {
-			(void) file_finish_load (wbc, new_wb);
-		} else {
+		if (!wb_view_open (wb_control_view (wbc), wbc, file_name, FALSE)) {
 #ifdef ENABLE_BONOBO
 			/* If it wasn't a workbook, see if we have a control for it */
 			SheetObject *so = sheet_object_container_new_file (
-			                  gsheet->scg->sheet, file_name);
+				sc->sheet, file_name);
 			if (so != NULL)
 				scg_mode_create_object (gsheet->scg, so);
 #else
@@ -394,7 +536,7 @@ gnumeric_sheet_filenames_dropped (GtkWidget        *widget,
 
 			msg = g_strdup_printf (_("File \"%s\" has unknown format."),
 			                       file_name);
-			gnumeric_error_read (COMMAND_CONTEXT (wbc), msg);
+			gnumeric_error_read (COMMAND_CONTEXT (sc->wbc), msg);
 			g_free (msg);
 #endif
 		}
@@ -404,6 +546,7 @@ gnumeric_sheet_filenames_dropped (GtkWidget        *widget,
 static void
 gnumeric_sheet_realize (GtkWidget *widget)
 {
+	gint width, height;
 	GdkWindow *window;
 	GnumericSheet *gsheet;
 
@@ -423,12 +566,40 @@ gnumeric_sheet_realize (GtkWidget *widget)
 		GdkIMStyle style;
 		GdkIMStyle supported_style = GDK_IM_PREEDIT_NONE |
 			GDK_IM_PREEDIT_NOTHING |
+			GDK_IM_PREEDIT_POSITION |
 			GDK_IM_STATUS_NONE |
 			GDK_IM_STATUS_NOTHING;
 
+		if(widget->style && widget->style->font->type != GDK_FONT_FONTSET)
+		  supported_style &= ~GDK_IM_PREEDIT_POSITION;
+
 		attr->style = style = gdk_im_decide_style (supported_style);
 		attr->client_window = gsheet->canvas.layout.bin_window;
 
+		switch (style & GDK_IM_PREEDIT_MASK)
+		  {
+		  case GDK_IM_PREEDIT_POSITION:
+		    if (widget->style && widget->style->font->type != GDK_FONT_FONTSET)
+		      {
+		        g_warning ("over-the-spot style requires fontset");			        break;
+		      }
+
+		    gdk_window_get_size (attr->client_window, &width, &height);
+		    height = widget->style->font->ascent +
+		             widget->style->font->descent;
+
+		    attrmask |= GDK_IC_PREEDIT_POSITION_REQ;
+		    attr->spot_location.x = 0;
+		    attr->spot_location.y = height;
+		    attr->preedit_area.x = 0;
+		    attr->preedit_area.y = 0;
+		    attr->preedit_area.width = width;
+		    attr->preedit_area.height = height;
+		    attr->preedit_fontset = widget->style->font;
+
+		    break;
+		  }
+
 		gsheet->ic = gdk_ic_new (attr, attrmask);
 		if (gsheet->ic != NULL) {
 			mask = gdk_window_get_events (attr->client_window);
@@ -507,36 +678,28 @@ gnumeric_sheet_init (GnumericSheet *gsheet)
 
 	gsheet->ic = NULL;
 	gsheet->ic_attr = NULL;
+	gsheet->col.first = gsheet->col.last_full = gsheet->col.last_visible = 0;
+	gsheet->row.first = gsheet->row.last_full = gsheet->row.last_visible = 0;
+	gsheet->col_offset.first = gsheet->col_offset.last_full = gsheet->col_offset.last_visible = 0;
+	gsheet->row_offset.first = gsheet->row_offset.last_full = gsheet->row_offset.last_visible = 0;
+
+	gsheet->slide_handler = NULL;
+	gsheet->slide_data = NULL;
+	gsheet->sliding = -1;
+	gsheet->sliding_x  = gsheet->sliding_dx = -1;
+	gsheet->sliding_y  = gsheet->sliding_dy = -1;
+	gsheet->sliding_adjacent_h = gsheet->sliding_adjacent_v = FALSE;
 
 	GTK_WIDGET_SET_FLAGS (canvas, GTK_CAN_FOCUS);
 	GTK_WIDGET_SET_FLAGS (canvas, GTK_CAN_DEFAULT);
 }
 
-GtkType
-gnumeric_sheet_get_type (void)
-{
-	static GtkType gnumeric_sheet_type = 0;
+E_MAKE_TYPE (gnumeric_sheet, "GnumericSheet", GnumericSheet,
+	     gnumeric_sheet_class_init, gnumeric_sheet_init,
+	     GNOME_TYPE_CANVAS);
 
-	if (!gnumeric_sheet_type) {
-		GtkTypeInfo gnumeric_sheet_info = {
-			"GnumericSheet",
-			sizeof (GnumericSheet),
-			sizeof (GnumericSheetClass),
-			(GtkClassInitFunc) gnumeric_sheet_class_init,
-			(GtkObjectInitFunc) gnumeric_sheet_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		gnumeric_sheet_type = gtk_type_unique (gnome_canvas_get_type (), &gnumeric_sheet_info);
-	}
-
-	return gnumeric_sheet_type;
-}
-
-GtkWidget *
-gnumeric_sheet_new (SheetControlGUI *scg)
+GnumericSheet *
+gnumeric_sheet_new (SheetControlGUI *scg, GnumericPane *pane)
 {
 	static GtkTargetEntry const drag_types[] = {
 		{ "text/uri-list", 0, 0 },
@@ -545,21 +708,16 @@ gnumeric_sheet_new (SheetControlGUI *scg)
 
 	GnomeCanvasItem	 *item;
 	GnumericSheet	 *gsheet;
-	GnomeCanvasGroup *gsheet_group;
-	GtkWidget	 *widget;
+	GnomeCanvasGroup *gsheet_group, *root_group;
 	Range		  r;
 
 	g_return_val_if_fail (IS_SHEET_CONTROL_GUI (scg), NULL);
 
 	gsheet = gtk_type_new (gnumeric_sheet_get_type ());
 	gsheet_group = GNOME_CANVAS_GROUP (GNOME_CANVAS (gsheet)->root);
-	widget = GTK_WIDGET (gsheet);
 
 	gsheet->scg = scg;
-	gsheet->row.first = gsheet->row.last_full = gsheet->row.last_visible = 0;
-	gsheet->col.first = gsheet->col.last_full = gsheet->col.last_visible = 0;
-	gsheet->row_offset.first = gsheet->row_offset.last_full = gsheet->row_offset.last_visible = 0;
-	gsheet->col_offset.first = gsheet->col_offset.last_full = gsheet->col_offset.last_visible = 0;
+	gsheet->pane = pane;
 
 	/* FIXME: figure out some real size for the canvas scrolling region */
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (gsheet), 0, 0,
@@ -581,105 +739,34 @@ gnumeric_sheet_new (SheetControlGUI *scg)
 	gnumeric_sheet_cursor_bound (gsheet, range_init (&r, 0, 0, 0, 0)); /* A1 */
 
 	/* Setup a test of Drag and Drop */
-	gtk_signal_connect (GTK_OBJECT (widget),
+	gtk_signal_connect (GTK_OBJECT (gsheet),
 		"drag_data_get",
 		GTK_SIGNAL_FUNC (gnumeric_sheet_drag_data_get), NULL);
-	gtk_drag_dest_set (widget,
+	gtk_drag_dest_set (GTK_WIDGET (gsheet),
 		GTK_DEST_DEFAULT_ALL,
 		drag_types, n_drag_types,
 		GDK_ACTION_COPY);
-	gtk_signal_connect (GTK_OBJECT (widget),
+	gtk_signal_connect (GTK_OBJECT (gsheet),
 		"drag_data_received",
 		GTK_SIGNAL_FUNC (gnumeric_sheet_filenames_dropped),
-		widget);
+		gsheet);
 
-	return widget;
-}
+	/* cursor group */
+	root_group = GNOME_CANVAS_GROUP (GNOME_CANVAS (gsheet)->root);
+	gsheet->anted_group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (
+			root_group, gnome_canvas_group_get_type (),
+			"x", 0.0,
+			"y", 0.0,
+			NULL));
 
-static int
-gnumeric_sheet_bar_set_top_row (GnumericSheet *gsheet, int new_first_row)
-{
-	GnomeCanvas *rowc;
-	int row_offset;
-	int x;
+	/* sheet object support */
+	gsheet->object_group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (
+			root_group, gnome_canvas_group_get_type (),
+			"x", 0.0,
+			"y", 0.0,
+			NULL));
 
-	g_return_val_if_fail (gsheet != NULL, 0);
-	g_return_val_if_fail (gsheet->item_grid != NULL, 0);
-	g_return_val_if_fail (0 <= new_first_row && new_first_row < SHEET_MAX_ROWS, 0);
-
-	rowc = gsheet->scg->row_item->canvas;
-	row_offset = gsheet->row_offset.first +=
-		scg_colrow_distance_get (gsheet->scg, FALSE, gsheet->row.first, new_first_row);
-	gsheet->row.first = new_first_row;
-
-	/* Scroll the row headers */
-	gnome_canvas_get_scroll_offsets (rowc, &x, NULL);
-	gnome_canvas_scroll_to (rowc, x, row_offset);
-
-	return row_offset;
-}
-
-void
-gnumeric_sheet_set_top_row (GnumericSheet *gsheet, int new_first_row)
-{
-	g_return_if_fail (gsheet != NULL);
-	g_return_if_fail (0 <= new_first_row && new_first_row < SHEET_MAX_ROWS);
-
-	if (gsheet->row.first != new_first_row) {
-		int x;
-		GnomeCanvas * const canvas = GNOME_CANVAS(gsheet);
-		int const y_offset =
-			gnumeric_sheet_bar_set_top_row (gsheet, new_first_row);
-
-		gsheet_compute_visible_region (gsheet, FALSE);
-
-		/* Scroll the cell canvas */
-		gnome_canvas_get_scroll_offsets (canvas, &x, NULL);
-		gnome_canvas_scroll_to (canvas, x, y_offset);
-	}
-}
-
-static int
-gnumeric_sheet_bar_set_left_col (GnumericSheet *gsheet, int new_first_col)
-{
-	GnomeCanvas *colc;
-	int col_offset;
-	int y;
-
-	g_return_val_if_fail (gsheet != NULL, 0);
-	g_return_val_if_fail (gsheet->item_grid != NULL, 0);
-	g_return_val_if_fail (0 <= new_first_col && new_first_col < SHEET_MAX_COLS, 0);
-
-	colc = gsheet->scg->col_item->canvas;
-	col_offset = gsheet->col_offset.first +=
-		scg_colrow_distance_get (gsheet->scg, TRUE, gsheet->col.first, new_first_col);
-	gsheet->col.first = new_first_col;
-
-	/* Scroll the column headers */
-	gnome_canvas_get_scroll_offsets (colc, NULL, &y);
-	gnome_canvas_scroll_to (colc, col_offset, y);
-
-	return col_offset;
-}
-
-void
-gnumeric_sheet_set_left_col (GnumericSheet *gsheet, int new_first_col)
-{
-	g_return_if_fail (gsheet != NULL);
-	g_return_if_fail (0 <= new_first_col && new_first_col < SHEET_MAX_COLS);
-
-	if (gsheet->col.first != new_first_col) {
-		int y;
-		GnomeCanvas * const canvas = GNOME_CANVAS(gsheet);
-		int const x_offset =
-			gnumeric_sheet_bar_set_left_col (gsheet, new_first_col);
-
-		gsheet_compute_visible_region (gsheet, FALSE);
-
-		/* Scroll the cell canvas */
-		gnome_canvas_get_scroll_offsets (canvas, NULL, &y);
-		gnome_canvas_scroll_to (canvas, x_offset, y);
-	}
+	return gsheet;
 }
 
 /*
@@ -688,7 +775,7 @@ gnumeric_sheet_set_left_col (GnumericSheet *gsheet, int new_first_col)
 int
 gnumeric_sheet_find_col (GnumericSheet *gsheet, int x, int *col_origin)
 {
-	Sheet *sheet = gsheet->scg->sheet;
+	Sheet *sheet = ((SheetControl *) gsheet->scg)->sheet;
 	int col   = gsheet->col.first;
 	int pixel = gsheet->col_offset.first;
 
@@ -732,7 +819,7 @@ gnumeric_sheet_find_col (GnumericSheet *gsheet, int x, int *col_origin)
 int
 gnumeric_sheet_find_row (GnumericSheet *gsheet, int y, int *row_origin)
 {
-	Sheet *sheet = gsheet->scg->sheet;
+	Sheet *sheet = ((SheetControl *) gsheet->scg)->sheet;
 	int row   = gsheet->row.first;
 	int pixel = gsheet->row_offset.first;
 
@@ -773,17 +860,31 @@ gnumeric_sheet_find_row (GnumericSheet *gsheet, int y, int *row_origin)
 void
 gnumeric_sheet_create_editor (GnumericSheet *gsheet)
 {
-	GnomeCanvas *canvas = GNOME_CANVAS (gsheet);
-	GnomeCanvasItem *item;
+	CellPos *edit_pos;
 
+	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
 	g_return_if_fail (gsheet->item_editor == NULL);
 
-	item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (canvas->root),
-				      item_edit_get_type (),
-				      "ItemEdit::SheetControlGUI",     gsheet->scg,
-				      NULL);
+	/* TODO : this could be slicker.
+	 * Rather than doing a visibility check here.
+	 * we could make item-edit smarter, and have it bound check on the
+	 * entire region rather than only its canvas.
+	 */
+	edit_pos = &(((SheetControl *) gsheet->scg)->sheet->edit_pos);
+	if (edit_pos->col >= gsheet->col.first &&
+	    edit_pos->col <= gsheet->col.last_visible &&
+	    edit_pos->row >= gsheet->row.first &&
+	    edit_pos->row <= gsheet->row.last_visible) {
+		GnomeCanvas *canvas = GNOME_CANVAS (gsheet);
+		GnomeCanvasItem *item;
 
-	gsheet->item_editor = ITEM_EDIT (item);
+		item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (canvas->root),
+			item_edit_get_type (),
+			"ItemEdit::SheetControlGUI",     gsheet->scg,
+			NULL);
+
+		gsheet->item_editor = ITEM_EDIT (item);
+	}
 }
 
 void
@@ -817,8 +918,7 @@ gnumeric_sheet_rangesel_start (GnumericSheet *gsheet, int col, int row)
 	GnomeCanvas *canvas = GNOME_CANVAS (gsheet);
 	GnomeCanvasItem *tmp;
 	GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (canvas->root);
-	Sheet *sheet = gsheet->scg->sheet;
-	WorkbookControlGUI *wbcg = gsheet->scg->wbcg;
+	SheetControl *sc = (SheetControl *) gsheet->scg;
 	Range r;
 
 	g_return_if_fail (gsheet->sel_cursor == NULL);
@@ -826,7 +926,7 @@ gnumeric_sheet_rangesel_start (GnumericSheet *gsheet, int col, int row)
 	/* Hide the primary cursor while the range selection cursor is visible
 	 * and we are selecting on a different sheet than the expr being edited
 	 */
-	if (sheet != wb_control_cur_sheet (WORKBOOK_CONTROL (wbcg)))
+	if (sc->sheet != wb_control_cur_sheet (sc->wbc))
 		item_cursor_set_visibility (gsheet->item_cursor, FALSE);
 
 	tmp = gnome_canvas_item_new (group,
@@ -857,6 +957,8 @@ gnumeric_sheet_rangesel_stop (GnumericSheet *gsheet)
 
 	/* Make the primary cursor visible again */
 	item_cursor_set_visibility (gsheet->item_cursor, TRUE);
+
+	gnumeric_sheet_slide_stop (gsheet);
 }
 
 /*
@@ -872,20 +974,23 @@ gsheet_compute_visible_region (GnumericSheet *gsheet,
 			       gboolean const full_recompute)
 {
 	SheetControlGUI const * const scg = gsheet->scg;
-	Sheet const * const sheet = scg->sheet;
+	Sheet const * const sheet = ((SheetControl *) scg)->sheet;
 	GnomeCanvas   *canvas = GNOME_CANVAS (gsheet);
 	int pixels, col, row, width, height;
 
 	/* When col/row sizes change we need to do a full recompute */
 	if (full_recompute) {
-		gsheet->col_offset.first =
-			scg_colrow_distance_get (scg, TRUE, 0, gsheet->col.first);
-		gnome_canvas_scroll_to (scg->col_item->canvas,
-			gsheet->col_offset.first, 0);
-		gsheet->row_offset.first =
-			scg_colrow_distance_get (scg, FALSE, 0, gsheet->row.first);
-		gnome_canvas_scroll_to (scg->row_item->canvas,
-			0, gsheet->row_offset.first);
+		gsheet->col_offset.first = scg_colrow_distance_get (scg,
+			TRUE, 0, gsheet->col.first);
+		if (NULL != gsheet->pane->col.canvas)
+			gnome_canvas_scroll_to (gsheet->pane->col.canvas,
+				gsheet->col_offset.first, 0);
+
+		gsheet->row_offset.first = scg_colrow_distance_get (scg,
+			FALSE, 0, gsheet->row.first);
+		if (NULL != gsheet->pane->row.canvas)
+			gnome_canvas_scroll_to (gsheet->pane->row.canvas,
+				0, gsheet->row_offset.first);
 
 		gnome_canvas_scroll_to (GNOME_CANVAS (gsheet),
 					gsheet->col_offset.first,
@@ -957,114 +1062,412 @@ gsheet_compute_visible_region (GnumericSheet *gsheet,
 		gsheet->row.last_full = SHEET_MAX_ROWS-1;
 	}
 
-	/* Update the scrollbar sizes */
-	scg_scrollbar_config (scg);
+	/* Update the scrollbar sizes for the primary pane */
+	if (gsheet->pane->index == 0)
+		scg_scrollbar_config (SHEET_CONTROL (scg));
 
 	/* Force the cursor to update its bounds relative to the new visible region */
+	/* FIXME : Shouldn't we do the animated cursors too ?
+	 * in sheet_update_only_grid we manually call update_cursors after
+	 * every call here.
+	 */
 	item_cursor_reposition (gsheet->item_cursor);
 }
 
-/**
- * gnumeric_sheet_make_cell_visible
- * @gsheet        sheet widget
- * @col           column
- * @row           row
- * @force_scroll  force a scroll
- *
- * Ensure that cell (col, row) is visible.
- * Sheet is scrolled if cell is outside viewport.
- *
- * Avoid calling this before the canvas is realized:
- * We do not know the visible area, and would unconditionally scroll the cell
- * to the top left of the viewport.
- */
 void
-gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row,
-				  gboolean const force_scroll)
+gnumeric_sheet_redraw_region (GnumericSheet *gsheet,
+			      int start_col, int start_row,
+			      int end_col, int end_row)
 {
+	SheetControlGUI *scg;
 	GnomeCanvas *canvas;
-	Sheet *sheet;
-	int   did_change = 0;
-	int   new_first_col, new_first_row;
-	int   col_offset, row_offset;
+	int x1, y1, x2, y2;
 
 	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
-	g_return_if_fail (col >= 0);
-	g_return_if_fail (row >= 0);
-	g_return_if_fail (col < SHEET_MAX_COLS);
-	g_return_if_fail (row < SHEET_MAX_ROWS);
 
-	sheet = gsheet->scg->sheet;
+	scg = gsheet->scg;
 	canvas = GNOME_CANVAS (gsheet);
 
-	/* Find the new gsheet->col.first */
-	if (col < gsheet->col.first) {
-		new_first_col = col;
-	} else if (col > gsheet->col.last_full) {
-		int width = GTK_WIDGET (canvas)->allocation.width;
-		int first_col = (gsheet->col.last_visible == gsheet->col.first)
-			? gsheet->col.first : col;
-
-		for (; first_col > 0; --first_col) {
-			ColRowInfo const * const ci = sheet_col_get_info (sheet, first_col);
-			if (ci->visible) {
-				width -= ci->size_pixels;
-				if (width < 0)
-					break;
-			}
-		}
-		new_first_col = first_col+1;
-	} else
-		new_first_col = gsheet->col.first;
-
-	/* Find the new gsheet->row.first */
-	if (row < gsheet->row.first) {
-		new_first_row = row;
-	} else if (row > gsheet->row.last_full) {
-		int height = GTK_WIDGET (canvas)->allocation.height;
-		int first_row = (gsheet->row.last_visible == gsheet->row.first)
-			? gsheet->row.first : row;
-
-		for (; first_row > 0; --first_row) {
-			ColRowInfo const * const ri = sheet_row_get_info (sheet, first_row);
-			if (ri->visible) {
-				height -= ri->size_pixels;
-				if (height < 0)
-					break;
-			}
-		}
-		new_first_row = first_row+1;
-	} else
-		new_first_row = gsheet->row.first;
-
-	/* Determine if scrolling is required */
-	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gsheet), &col_offset, &row_offset);
-
-	if (gsheet->col.first != new_first_col || force_scroll) {
-		if (force_scroll) {
-			/* Clear the offsets in case col/row size changed */
-			gsheet->col_offset.first = 0;
-			gsheet->col.first = 0;
-		}
-		col_offset = gnumeric_sheet_bar_set_left_col (gsheet, new_first_col);
-		did_change = 1;
-	}
-
-	if (gsheet->row.first != new_first_row || force_scroll) {
-		if (force_scroll) {
-			/* Clear the offsets in case col/row size changed */
-			gsheet->row_offset.first = 0;
-			gsheet->row.first = 0;
-		}
-		row_offset = gnumeric_sheet_bar_set_top_row (gsheet, new_first_row);
-		did_change = 1;
-	}
-
-	if (!did_change && !force_scroll)
+	if ((end_col < gsheet->col.first) ||
+	    (end_row < gsheet->row.first) ||
+	    (start_col > gsheet->col.last_visible) ||
+	    (start_row > gsheet->row.last_visible))
 		return;
 
-	gsheet_compute_visible_region (gsheet, FALSE);
+	/* Only draw those regions that are visible */
+	start_col = MAX (gsheet->col.first, start_col);
+	start_row = MAX (gsheet->row.first, start_row);
+	end_col =  MIN (gsheet->col.last_visible, end_col);
+	end_row =  MIN (gsheet->row.last_visible, end_row);
 
-	gnome_canvas_scroll_to (GNOME_CANVAS (gsheet), col_offset, row_offset);
+	/* redraw a border of 2 pixels around the region to handle thick borders
+	 * NOTE the 2nd coordinates are excluded so add 1 extra (+2border +1include)
+	 */
+	x1 = scg_colrow_distance_get (scg, TRUE, gsheet->col.first, start_col) +
+		gsheet->col_offset.first;
+	y1 = scg_colrow_distance_get (scg, FALSE, gsheet->row.first, start_row) +
+		gsheet->row_offset.first;
+	x2 = (end_col < (SHEET_MAX_COLS-1))
+		? 4 + 1 + x1 + scg_colrow_distance_get (scg, TRUE,
+							start_col, end_col+1)
+		: INT_MAX;
+	y2 = (end_row < (SHEET_MAX_ROWS-1))
+		? 4 + 1 + y1 + scg_colrow_distance_get (scg, FALSE,
+							start_row, end_row+1)
+		: INT_MAX;
+
+#if 0
+	fprintf (stderr, "%s%s:", col_name (min_col), row_name (first_row));
+	fprintf (stderr, "%s%s\n", col_name (max_col), row_name (last_row));
+#endif
+
+	gnome_canvas_request_redraw (GNOME_CANVAS (gsheet), x1-2, y1-2, x2, y2);
 }
 
+/*****************************************************************************/
+
+void
+gnumeric_sheet_slide_stop (GnumericSheet *gsheet)
+{
+	if (gsheet->sliding == -1)
+		return;
+
+	gtk_timeout_remove (gsheet->sliding);
+	gsheet->slide_handler = NULL;
+	gsheet->slide_data = NULL;
+	gsheet->sliding = -1;
+}
+
+static int
+col_scroll_step (int dx)
+{
+	if (dx <= 30)
+		return 1;
+	if (dx <= 60)
+		return 2;
+	if (dx <= 90)
+		return 4;
+	if (dx <= 120)
+		return 8;
+	return 16;
+}
+
+static int
+row_scroll_step (int dy)
+{
+	if (dy <= 20)
+		return 1;
+	if (dy <= 30)
+		return 2;
+	if (dy <= 40)
+		return 4;
+	if (dy <= 45)
+		return 8;
+	if (dy <= 50)
+		return 16;
+	if (dy <= 60)
+		return 32;
+	if (dy <= 100)
+		return 500;
+	return 5000;
+}
+
+static gint
+gsheet_sliding_callback (gpointer data)
+{
+	GnumericSheet *gsheet = data;
+	int const pane_index = gsheet->pane->index;
+	Sheet const *sheet = sc_sheet (SHEET_CONTROL (gsheet->scg));
+	GnumericSheet *gsheet0 = scg_pane (gsheet->scg, 0);
+	GnumericSheet *gsheet2 = sheet_is_frozen (sheet)
+		? scg_pane (gsheet->scg, 2) : NULL;
+	gboolean slide_x = FALSE, slide_y = FALSE;
+	int col = -1, row = -1;
+
+	if (gsheet->sliding_dx > 0) {
+		GnumericSheet *target_gsheet = gsheet;
+
+		slide_x = TRUE;
+		if (pane_index == 1 || pane_index == 2) {
+			if (!gsheet->sliding_adjacent_h) {
+				int width = GTK_WIDGET (gsheet)->allocation.width;
+				int x = gsheet->col_offset.first + width + gsheet->sliding_dx;
+
+				/* in case pane is narrow */
+				col = gnumeric_sheet_find_col (gsheet, x, NULL);
+				if (col > gsheet0->col.last_full) {
+					gsheet->sliding_adjacent_h = TRUE;
+					gsheet->sliding_dx = 1; /* good enough */
+				} else
+					slide_x = FALSE;
+			} else
+				target_gsheet = gsheet0;
+		}
+
+		if (slide_x) {
+			col = target_gsheet->col.last_full + 
+				col_scroll_step (gsheet->sliding_dx);
+			if (col >= SHEET_MAX_COLS-1) {
+				col = SHEET_MAX_COLS-1;
+				slide_x = FALSE;
+			}
+		}
+	} else if (gsheet->sliding_dx < 0) {
+		slide_x = TRUE;
+		col = gsheet0->col.first - col_scroll_step (-gsheet->sliding_dx);
+
+		if (gsheet2 != NULL) {
+			if (pane_index == 0 || pane_index == 3) {
+				int width = GTK_WIDGET (gsheet2)->allocation.width;
+				if (gsheet->sliding_dx > (-width) &&
+				    col <= gsheet2->col.last_visible) {
+					int x = gsheet2->col_offset.first + width + gsheet->sliding_dx;
+					col = gnumeric_sheet_find_col (gsheet2, x, NULL);
+					slide_x = FALSE;
+				}
+			} 
+
+			if (col <= gsheet2->col.first) {
+				col = gsheet2->col.first;
+				slide_x = FALSE;
+			}
+		} else if (col <= 0) {
+			col = 0;
+			slide_x = FALSE;
+		}
+	} 
+
+	if (gsheet->sliding_dy > 0) {
+		GnumericSheet *target_gsheet = gsheet;
+
+		slide_y = TRUE;
+		if (pane_index == 3 || pane_index == 2) {
+			if (!gsheet->sliding_adjacent_v) {
+				int height = GTK_WIDGET (gsheet)->allocation.height;
+				int y = gsheet->row_offset.first + height + gsheet->sliding_dy;
+
+				/* in case pane is short */
+				row = gnumeric_sheet_find_row (gsheet, y, NULL);
+				if (row > gsheet0->row.last_full) {
+					gsheet->sliding_adjacent_v = TRUE;
+					gsheet->sliding_dy = 1; /* good enough */
+				} else
+					slide_y = FALSE;
+			} else
+				target_gsheet = gsheet0;
+		}
+
+		if (slide_y) {
+			row = target_gsheet->row.last_full + 
+				row_scroll_step (gsheet->sliding_dy);
+			if (row >= SHEET_MAX_ROWS-1) {
+				row = SHEET_MAX_ROWS-1;
+				slide_y = FALSE;
+			}
+		}
+	} else if (gsheet->sliding_dy < 0) {
+		slide_y = TRUE;
+		row = gsheet0->row.first - row_scroll_step (-gsheet->sliding_dy);
+
+		if (gsheet2 != NULL) {
+			if (pane_index == 0 || pane_index == 1) {
+				int height = GTK_WIDGET (gsheet2)->allocation.height;
+				if (gsheet->sliding_dy > (-height) &&
+				    row <= gsheet2->row.last_visible) {
+					int y = gsheet2->row_offset.first + height + gsheet->sliding_dy;
+					row = gnumeric_sheet_find_row (gsheet2, y, NULL);
+					slide_y = FALSE;
+				}
+			} 
+
+			if (row <= gsheet2->row.first) {
+				row = gsheet2->row.first;
+				slide_y = FALSE;
+			}
+		} else if (row <= 0) {
+			row = 0;
+			slide_y = FALSE;
+		}
+	}
+
+	if (col < 0 && row < 0) {
+		gnumeric_sheet_slide_stop (gsheet);
+		return TRUE;
+	}
+
+	if (col < 0)
+		col = gnumeric_sheet_find_col (gsheet, gsheet->sliding_x, NULL);
+	else if (row < 0)
+		row = gnumeric_sheet_find_row (gsheet, gsheet->sliding_y, NULL);
+
+	if (gsheet->slide_handler == NULL ||
+	    (*gsheet->slide_handler) (gsheet, col, row, gsheet->slide_data))
+		scg_make_cell_visible (gsheet->scg, col, row, FALSE, TRUE);
+
+	if (slide_x || slide_y) {
+		if (gsheet->sliding == -1)
+			gsheet->sliding = gtk_timeout_add (
+				300, gsheet_sliding_callback, gsheet);
+	} else
+		gnumeric_sheet_slide_stop (gsheet);
+
+	return TRUE;
+}
+
+/**
+ * gnumeric_sheet_handle_motion :
+ * @gsheet	 : The GnumericSheet managing the scroll
+ * @canvas	 : The Canvas the event comes from
+ * @event	 : The motion event
+ * @slide_flags	 : 
+ * @slide_handler: The handler when sliding
+ * @user_data	 : closure data
+ *
+ * Handle a motion event from a @canvas and scroll the @gsheet
+ * depending on how far outside the bounds of @gsheet the @event is.
+ * Usually @canvas == @gsheet however as long as the canvases share a basis
+ * space they can be different.
+ */
+gboolean
+gnumeric_sheet_handle_motion (GnumericSheet *gsheet,
+			      GnomeCanvas *canvas, GdkEventMotion *event,
+			      GnumericSlideFlags slide_flags,
+			      GnumericSheetSlideHandler slide_handler,
+			      gpointer user_data)
+{
+	Sheet const *sheet;
+	GnumericSheet *gsheet0;
+	int left, top, x, y, width, height;
+	int dx = 0, dy = 0;
+
+	g_return_val_if_fail (GNUMERIC_IS_SHEET (gsheet), FALSE);
+	g_return_val_if_fail (GNOME_IS_CANVAS (canvas), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+	g_return_val_if_fail (slide_handler != NULL, FALSE);
+
+	gnome_canvas_w2c (canvas, event->x, event->y, &x, &y);
+
+	left = gsheet->col_offset.first;
+	top = gsheet->row_offset.first;
+	width = GTK_WIDGET (gsheet)->allocation.width;
+	height = GTK_WIDGET (gsheet)->allocation.height;
+
+	sheet = sc_sheet (SHEET_CONTROL (gsheet->scg));
+	gsheet0 = scg_pane (gsheet->scg, 0);
+
+	if (slide_flags & GNM_SLIDE_X) {
+		if (x < left)
+			dx = x - left;
+		else if (x >= left + width)
+			dx = x - width - left;
+	}
+
+	if (slide_flags & GNM_SLIDE_Y) {
+		if (y < top)
+			dy = y - top;
+		else if (y >= top + height)
+			dy = y - height - top;
+	}
+
+	if (sheet_is_frozen (sheet)) {
+		int const pane = gsheet->pane->index;
+		GnumericSheet *gsheet2 = scg_pane (gsheet->scg, 2);
+
+		if (gsheet->sliding_adjacent_h) {
+			if (pane == 0 || pane == 3) {
+				if (dx < 0) {
+					dx += GTK_WIDGET (gsheet2)->allocation.width;
+					x = gsheet2->col_offset.first + dx;
+					if (dx > 0)
+						dx = 0;
+				}
+			} else {
+				if (dx > 0) {
+					x = gsheet0->col_offset.first + dx;
+					dx -= GTK_WIDGET (gsheet0)->allocation.width;
+					if (dx < 0)
+						dx = 0;
+				} else if (dx == 0) {
+					/* initiate a reverse scroll of panes 0,3 */
+					if ((gsheet2->col.last_visible+1) != gsheet0->col.first)
+						dx = x - (left + width);
+				} else
+					dx = 0;
+			}
+		}
+
+		if (gsheet->sliding_adjacent_v) {
+			if (pane == 0 || pane == 1) {
+				if (dy < 0) {
+					dy += GTK_WIDGET (gsheet2)->allocation.height;
+					y = gsheet2->row_offset.first +  dy;
+					if (dy > 0)
+						dy = 0;
+				}
+			} else {
+				if (dy > 0) {
+					y = gsheet0->row_offset.first + dy;
+					dy -= GTK_WIDGET (gsheet0)->allocation.height;
+					if (dy < 0)
+						dy = 0;
+				} else if (dy == 0) {
+					/* initiate a reverse scroll of panes 0,1 */
+					if ((gsheet2->row.last_visible+1) != gsheet0->row.first)
+						dy = y - (top + height);
+				} else
+					dy = 0;
+			}
+		}
+	}
+	/* Movement is inside the visible region */
+	if (dx == 0 && dy == 0) {
+		if (!(slide_flags & GNM_SLIDE_EXTERIOR_ONLY)) {
+			int const col = gnumeric_sheet_find_col (gsheet, x, NULL);
+			int const row = gnumeric_sheet_find_row (gsheet, y, NULL);
+
+			(*slide_handler) (gsheet, col, row, user_data);
+		}
+		gnumeric_sheet_slide_stop (gsheet);
+		return TRUE;
+	}
+
+	gsheet->sliding_x  = x;
+	gsheet->sliding_dx = dx;
+	gsheet->sliding_y  = y;
+	gsheet->sliding_dy = dy;
+	gsheet->slide_handler = slide_handler;
+	gsheet->slide_data = user_data;
+
+	if (gsheet->sliding == -1)
+		(void) gsheet_sliding_callback (gsheet);
+	return FALSE;
+}
+
+/* TODO : All the slide_* members of GnumericSheet really aught to be in
+ * SheetControlGUI, most of these routines also belong there.  However, since
+ * the primary point of access is via the GSheet and SCG is very large already
+ * I'm leaving them here for now.  Move them when we return to investigate
+ * how to do reverse scrolling for pseudo-adjacent panes.
+ */
+void
+gnumeric_sheet_slide_init (GnumericSheet *gsheet)
+{
+	Sheet const *sheet;
+	GnumericSheet *gsheet0;
+
+	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
+
+	sheet = sc_sheet (SHEET_CONTROL (gsheet->scg));
+	gsheet0 = scg_pane (gsheet->scg, 0);
+
+	if (sheet_is_frozen (sheet)) {
+		GnumericSheet const *gsheet2 = scg_pane (gsheet->scg, 2);
+
+		gsheet->sliding_adjacent_h =
+			gsheet2->col.last_full == (gsheet0->col.first - 1);
+		gsheet->sliding_adjacent_v =
+			gsheet2->row.last_full == (gsheet0->row.first - 1);
+	} else
+		gsheet->sliding_adjacent_h = gsheet->sliding_adjacent_v = FALSE;
+
+}
