@@ -350,10 +350,11 @@ parse_string_as_value_or_name (ExprTree *str)
 }
 
 static int
-gnumeric_parse_error (ParserState *state, char *message, int end, int relative_begin)
+gnumeric_parse_error (ParserState *state, ParseErrorID id, char *message, int end, int relative_begin)
 {
 	g_return_val_if_fail (state->error != NULL, ERROR);
 
+	state->error->id         = id;
 	state->error->message    = message;
 	state->error->begin_char = (end - relative_begin);
 	state->error->end_char   = end;
@@ -365,7 +366,7 @@ gnumeric_parse_error (ParserState *state, char *message, int end, int relative_b
 int yyparse (void);
 
 
-#line 348 "parser.y"
+#line 349 "parser.y"
 typedef union {
 	ExprTree *tree;
 	CellRef  *cell;
@@ -449,11 +450,11 @@ static const short yyrhs[] = {    35,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   371,   376,   384,   385,   386,   387,   388,   389,   390,   391,
-   392,   393,   394,   395,   396,   397,   398,   399,   400,   402,
-   403,   404,   406,   412,   431,   451,   452,   455,   470,   502,
-   503,   506,   510,   515,   526,   541,   546,   552,   562,   565,
-   566,   569,   574,   586,   599,   602,   607
+   372,   377,   385,   386,   387,   388,   389,   390,   391,   392,
+   393,   394,   395,   396,   397,   398,   399,   400,   401,   403,
+   404,   405,   407,   413,   432,   453,   454,   457,   473,   506,
+   507,   510,   514,   519,   530,   545,   550,   556,   566,   569,
+   570,   573,   578,   591,   605,   608,   613
 };
 #endif
 
@@ -1104,14 +1105,14 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 371 "parser.y"
+#line 372 "parser.y"
 {
 		unregister_allocation (yyvsp[0].tree);
 		state->result = yyvsp[0].tree;
 	;
     break;}
 case 2:
-#line 376 "parser.y"
+#line 377 "parser.y"
 {
 		if (state->result != NULL) {
 			expr_tree_unref (state->result);
@@ -1120,87 +1121,87 @@ case 2:
 	;
     break;}
 case 3:
-#line 384 "parser.y"
-{ yyval.tree = yyvsp[0].tree; ;
-    break;}
-case 4:
 #line 385 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
-case 5:
+case 4:
 #line 386 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
+    break;}
+case 5:
+#line 387 "parser.y"
 { yyval.tree = parse_string_as_value_or_name (yyvsp[0].tree); ;
     break;}
 case 6:
-#line 387 "parser.y"
+#line 388 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 7:
-#line 388 "parser.y"
+#line 389 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_ADD,       yyvsp[0].tree); ;
     break;}
 case 8:
-#line 389 "parser.y"
+#line 390 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_SUB,       yyvsp[0].tree); ;
     break;}
 case 9:
-#line 390 "parser.y"
+#line 391 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_MULT,      yyvsp[0].tree); ;
     break;}
 case 10:
-#line 391 "parser.y"
+#line 392 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_DIV,       yyvsp[0].tree); ;
     break;}
 case 11:
-#line 392 "parser.y"
+#line 393 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EXP,       yyvsp[0].tree); ;
     break;}
 case 12:
-#line 393 "parser.y"
+#line 394 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_CONCAT,    yyvsp[0].tree); ;
     break;}
 case 13:
-#line 394 "parser.y"
+#line 395 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EQUAL,     yyvsp[0].tree); ;
     break;}
 case 14:
-#line 395 "parser.y"
+#line 396 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LT,        yyvsp[0].tree); ;
     break;}
 case 15:
-#line 396 "parser.y"
+#line 397 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GT,        yyvsp[0].tree); ;
     break;}
 case 16:
-#line 397 "parser.y"
+#line 398 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GTE,       yyvsp[0].tree); ;
     break;}
 case 17:
-#line 398 "parser.y"
+#line 399 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_NOT_EQUAL, yyvsp[0].tree); ;
     break;}
 case 18:
-#line 399 "parser.y"
+#line 400 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LTE,       yyvsp[0].tree); ;
     break;}
 case 19:
-#line 400 "parser.y"
+#line 401 "parser.y"
 { yyval.tree = yyvsp[-1].tree; ;
     break;}
 case 20:
-#line 402 "parser.y"
+#line 403 "parser.y"
 { yyval.tree = build_unary_op (OPER_PERCENT, yyvsp[-1].tree); ;
     break;}
 case 21:
-#line 403 "parser.y"
+#line 404 "parser.y"
 { yyval.tree = build_unary_op (OPER_UNARY_NEG, yyvsp[0].tree); ;
     break;}
 case 22:
-#line 404 "parser.y"
+#line 405 "parser.y"
 { yyval.tree = build_unary_op (OPER_UNARY_PLUS, yyvsp[0].tree); ;
     break;}
 case 23:
-#line 406 "parser.y"
+#line 407 "parser.y"
 {
 		unregister_allocation (yyvsp[-1].list);
 		yyval.tree = build_array (yyvsp[-1].list);
@@ -1208,7 +1209,7 @@ case 23:
 	;
     break;}
 case 24:
-#line 412 "parser.y"
+#line 413 "parser.y"
 {
 		char const *name = yyvsp[-3].tree->constant.value->v_str.val->str;
 		FunctionDefinition *f = func_lookup_by_name (name,
@@ -1230,7 +1231,7 @@ case 24:
 	;
     break;}
 case 25:
-#line 431 "parser.y"
+#line 432 "parser.y"
 {
 		NamedExpression *expr_name;
 		char *name = yyvsp[0].tree->constant.value->v_str.val->str;
@@ -1240,7 +1241,8 @@ case 25:
 		expr_name = expr_name_lookup (&pos, name);
 		if (expr_name == NULL) {
 			int retval = gnumeric_parse_error (
-				state, g_strdup_printf (_("Expression '%s' does not exist on sheet '%s'"), name, yyvsp[-1].sheet->name_quoted),
+				state, PERR_UNKNOWN_EXPRESSION,
+				g_strdup_printf (_("Expression '%s' does not exist on sheet '%s'"), name, yyvsp[-1].sheet->name_quoted),
 				state->expr_text - state->expr_backup + 1, strlen (name));
 
 			unregister_allocation (yyvsp[0].tree); expr_tree_unref (yyvsp[0].tree);
@@ -1251,13 +1253,14 @@ case 25:
 	;
     break;}
 case 28:
-#line 455 "parser.y"
+#line 457 "parser.y"
 {
 	        char  *name = yyvsp[-1].tree->constant.value->v_str.val->str;
 		Sheet *sheet = sheet_lookup_by_name (state->pos->wb, name);
 		if (sheet == NULL) {
 			int retval = gnumeric_parse_error (
-				state, g_strdup_printf (_("Unknown sheet '%s'"), name),
+				state, PERR_UNKNOWN_SHEET,
+				g_strdup_printf (_("Unknown sheet '%s'"), name),
 				state->expr_text - state->expr_backup, strlen (name));
 
 			unregister_allocation (yyvsp[-1].tree); expr_tree_unref (yyvsp[-1].tree);
@@ -1268,7 +1271,7 @@ case 28:
 	;
     break;}
 case 29:
-#line 470 "parser.y"
+#line 473 "parser.y"
 {
 		/* TODO : Get rid of ParseErr and replace it with something richer.
 		 * The replace ment should include more detail as to what the error
@@ -1289,7 +1292,8 @@ case 29:
 		unregister_allocation (yyvsp[-3].tree); expr_tree_unref (yyvsp[-3].tree);
 		if (sheet == NULL) {
 			int retval = gnumeric_parse_error (
-				state, g_strdup_printf (_("Unknown sheet '%s'"), sheetname),
+				state, PERR_UNKNOWN_SHEET,
+				g_strdup_printf (_("Unknown sheet '%s'"), sheetname),
 				state->expr_text - state->expr_backup, strlen (sheetname));
 
 			unregister_allocation (yyvsp[-1].tree); expr_tree_unref (yyvsp[-1].tree);
@@ -1301,24 +1305,24 @@ case 29:
         ;
     break;}
 case 31:
-#line 503 "parser.y"
+#line 507 "parser.y"
 { yyval.sheet = NULL; ;
     break;}
 case 32:
-#line 506 "parser.y"
+#line 510 "parser.y"
 {
 	        yyval.tree = yyvsp[0].tree;
 	;
     break;}
 case 33:
-#line 510 "parser.y"
+#line 514 "parser.y"
 {
 		yyvsp[0].tree->var.ref.sheet = yyvsp[-1].sheet;
 	        yyval.tree = yyvsp[0].tree;
 	;
     break;}
 case 34:
-#line 515 "parser.y"
+#line 519 "parser.y"
 {
 		unregister_allocation (yyvsp[0].tree);
 		unregister_allocation (yyvsp[-2].tree);
@@ -1331,7 +1335,7 @@ case 34:
 	;
     break;}
 case 35:
-#line 526 "parser.y"
+#line 530 "parser.y"
 {
 		unregister_allocation (yyvsp[0].tree);
 		unregister_allocation (yyvsp[-3].tree);
@@ -1347,7 +1351,7 @@ case 35:
 	;
     break;}
 case 36:
-#line 541 "parser.y"
+#line 545 "parser.y"
 {
 		unregister_allocation (yyvsp[0].tree);
 		yyval.list = g_list_prepend (NULL, yyvsp[0].tree);
@@ -1355,7 +1359,7 @@ case 36:
         ;
     break;}
 case 37:
-#line 546 "parser.y"
+#line 550 "parser.y"
 {
 		unregister_allocation (yyvsp[0].list);
 		unregister_allocation (yyvsp[-2].tree);
@@ -1364,7 +1368,7 @@ case 37:
 	;
     break;}
 case 38:
-#line 552 "parser.y"
+#line 556 "parser.y"
 {
 		GList *tmp = yyvsp[0].list;
 		unregister_allocation (yyvsp[0].list);
@@ -1377,19 +1381,19 @@ case 38:
 	;
     break;}
 case 39:
-#line 562 "parser.y"
+#line 566 "parser.y"
 { yyval.list = NULL; ;
     break;}
 case 40:
-#line 565 "parser.y"
+#line 569 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 41:
-#line 566 "parser.y"
+#line 570 "parser.y"
 { parse_string_as_value (yyvsp[0].tree); yyval.tree = yyvsp[0].tree; ;
     break;}
 case 42:
-#line 569 "parser.y"
+#line 573 "parser.y"
 {
 		unregister_allocation (yyvsp[0].tree);
 		yyval.list = g_list_prepend (NULL, yyvsp[0].tree);
@@ -1397,7 +1401,7 @@ case 42:
         ;
     break;}
 case 43:
-#line 574 "parser.y"
+#line 578 "parser.y"
 {
 		if (state->array_col_separator == ',') {
 			unregister_allocation (yyvsp[0].list);
@@ -1406,13 +1410,14 @@ case 43:
 			register_expr_list_allocation (yyval.list);
 		} else {
 			return gnumeric_parse_error (
-				state, g_strdup_printf (_("The character %c can not be used to separate array elements"),
+				state, PERR_INVALID_ARRAY_SEPARATOR,
+				g_strdup_printf (_("The character %c can not be used to separate array elements"),
 				state->array_col_separator), state->expr_text - state->expr_backup + 1, 1);
 		}
 	;
     break;}
 case 44:
-#line 586 "parser.y"
+#line 591 "parser.y"
 {
 		if (state->array_col_separator == '\\') {
 			unregister_allocation (yyvsp[0].list);
@@ -1422,17 +1427,18 @@ case 44:
 		} else {
 			/* FIXME: Is this the right error to display? */
 			return gnumeric_parse_error (
-				state, g_strdup_printf (_("The character %c can not be used to separate array elements"),
+				state, PERR_INVALID_ARRAY_SEPARATOR,
+				g_strdup_printf (_("The character %c can not be used to separate array elements"),
 				state->array_col_separator), state->expr_text - state->expr_backup + 1, 1);
 		}
 	;
     break;}
 case 45:
-#line 599 "parser.y"
+#line 605 "parser.y"
 { yyval.list = NULL; ;
     break;}
 case 46:
-#line 602 "parser.y"
+#line 608 "parser.y"
 {
 		unregister_allocation (yyvsp[0].list);
 		yyval.list = g_list_prepend (NULL, yyvsp[0].list);
@@ -1440,7 +1446,7 @@ case 46:
         ;
     break;}
 case 47:
-#line 607 "parser.y"
+#line 613 "parser.y"
 {
 		unregister_allocation (yyvsp[0].list);
 		unregister_allocation (yyvsp[-2].list);
@@ -1670,7 +1676,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 614 "parser.y"
+#line 620 "parser.y"
 
 /**
  * parse_ref_or_string :
@@ -1707,7 +1713,7 @@ parse_ref_or_string (char const *string)
  * @c:
  *
  * Returns a pointer to character c in str.
- * Callers should check weather p is '\0'!
+ * Callers should check whether p is '\0'!
  **/
 static char const *
 find_char (char const *str, char c)
@@ -1738,7 +1744,6 @@ find_matching_close (char const *str, char const **res)
 	return str;
 }
 
-
 int
 yylex (void)
 {
@@ -1801,7 +1806,8 @@ yylex (void)
 				} else {
 					if (tolower (c) != 'e') {
 						gnumeric_parse_error (
-							state, g_strdup (_("The number is out of range")),
+							state, PERR_OUT_OF_RANGE,
+							g_strdup (_("The number is out of range")),
 							state->expr_text - state->expr_backup, end - start);
 						return INVALID_TOKEN;
 					} else {
@@ -1812,7 +1818,8 @@ yylex (void)
 						 * at the end.
 						 */
 						gnumeric_parse_error (
-							state, g_strdup (_("The number is out of range")),
+							state, PERR_OUT_OF_RANGE,
+							g_strdup (_("The number is out of range")),
 							0, 0);
 						return INVALID_TOKEN;
 					}
@@ -1839,7 +1846,8 @@ yylex (void)
 				} else {
 					if (l == LONG_MIN || l == LONG_MAX) {
 						gnumeric_parse_error (
-							state, g_strdup (_("The number is out of range")),
+							state, PERR_OUT_OF_RANGE,
+							g_strdup (_("The number is out of range")),
 							state->expr_text - state->expr_backup, end - start);
 						return INVALID_TOKEN;
 					}
@@ -1868,7 +1876,8 @@ yylex (void)
  		state->expr_text = find_char (state->expr_text, quotes_end);
 		if (!*state->expr_text) {
   			gnumeric_parse_error (
-  				state, g_strdup (_("Could not find matching closing quote")),
+  				state, PERR_MISSING_CLOSING_QUOTE,
+				g_strdup (_("Could not find matching closing quote")),
   				(p - state->expr_backup) + 1, 1);
 			return INVALID_TOKEN;
 		}
@@ -1967,7 +1976,7 @@ gnumeric_expr_parser (char const *expr_text, ParsePos const *pos,
 		*pstate.desired_format = NULL;
 
 	pstate.error = error;
-
+	
 	if (deallocate_stack == NULL)
 		deallocate_init ();
 
@@ -2014,20 +2023,20 @@ gnumeric_expr_parser (char const *expr_text, ParsePos const *pos,
 				char const *last = find_matching_close (str, &res);
 
 				if (*last)
-					gnumeric_parse_error (&pstate,
+					gnumeric_parse_error (&pstate, PERR_MISSING_PAREN_OPEN,
 						g_strdup (_("Could not find matching opening parenthesis")),
 						(last - str) + 2, 1);
 				else if (res != NULL)
-					gnumeric_parse_error (&pstate,
+					gnumeric_parse_error (&pstate, PERR_MISSING_PAREN_CLOSE,
 						g_strdup (_("Could not find matching closing parenthesis")),
 						(res - str) + 2, 1);
 				else
-					gnumeric_parse_error (&pstate,
+					gnumeric_parse_error (&pstate, PERR_INVALID_EXPRESSION,
 						g_strdup (_("Invalid expression")),
 						(pstate.expr_text - pstate.expr_backup) + 1,
 						(pstate.expr_text - pstate.expr_backup));
 			} else
-				gnumeric_parse_error (&pstate,
+				gnumeric_parse_error (&pstate, PERR_UNEXPECTED_TOKEN,
 					g_strdup_printf (_("Unexpected token %c"), *last_token),
 					(last_token - pstate.expr_backup) + 1, 1);
 		}
