@@ -10,26 +10,27 @@
 #include <bonobo/bonobo-ui-component.h>
 #include "gnumeric.h"
 #include "gnumeric-util.h"
-#include "gnumeric-type-util.h"
-#include "sheet-control-gui.h"
+#include "gnumeric-sheet.h"
+#include "sheet-control-gui-priv.h"
 #include "workbook-control-gui-priv.h"
 #include "dialogs.h"
 #include "sheet-object-item.h"
 
+#include <gal/util/e-util.h>
+
 static SheetObject *sheet_object_item_parent_class;
 
 static GtkObject *
-sheet_object_item_new_view (SheetObject *so, SheetControlGUI *s_control)
+sheet_object_item_new_view (SheetObject *so, SheetControlGUI *scg)
 {
+	/* FIXME : this is bogus */
+	GnumericSheet *gsheet = scg_pane (scg, 0);
 	GnomeCanvasItem *so_view = NULL;
 
-	/*
-	 * Create item/view-frame
-	 */
 	so_view = bonobo_client_site_new_item (
 		SHEET_OBJECT_BONOBO (so)->client_site,
-		bonobo_ui_component_get_container (s_control->wbcg->uic),
-		s_control->object_group);
+		bonobo_ui_component_get_container (scg->wbcg->uic),
+		gsheet->object_group);
 
 	scg_object_register (so, so_view);
 	return GTK_OBJECT (so_view);
@@ -53,6 +54,6 @@ sheet_object_item_class_init (GtkObjectClass *object_class)
 	sheet_object_class->update_bounds = sheet_object_item_update_bounds;
 }
 
-GNUMERIC_MAKE_TYPE (sheet_object_item, "SheetObjectItem", SheetObjectItem,
-		    sheet_object_item_class_init, NULL,
-		    sheet_object_bonobo_get_type ())
+E_MAKE_TYPE (sheet_object_item, "SheetObjectItem", SheetObjectItem,
+	     sheet_object_item_class_init, NULL,
+	     SHEET_OBJECT_BONOBO_TYPE);
