@@ -2,8 +2,7 @@
  * stf.c : Utilizes the stf-parse engine and the dialog-stf to provide a plug-in for
  *         importing text files with a structure (CSV/fixed width)
  *
- * Copyright (C) Almer. S. Tigelaar.
- * EMail: almer1@dds.nl or almer-t@bigfoot.com
+ * Copyright (C) Almer. S. Tigelaar <almer@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,19 +20,7 @@
  */
 
 #include <config.h>
-#include <stdio.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/mman.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
-#include <errno.h>
-#include <gnome.h>
-#include <glade/glade.h>
-#include <sys/types.h>
-
+#include "stf.h"
 #include "gnumeric.h"
 #include "plugin-util.h"
 #include "file.h"
@@ -47,12 +34,22 @@
 #include "workbook-control.h"
 #include "workbook-view.h"
 #include "workbook.h"
-#include "xml-io.h"
-
-#include "stf.h"
 #include "dialog-stf.h"
 #include "dialog-stf-export.h"
 
+#include <stdio.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/mman.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <ctype.h>
+#include <string.h>
+#include <errno.h>
+#include <gnome.h>
+#include <glade/glade.h>
+#include <sys/types.h>
+
 /**
  * stf_open_and_read
  * @filename : name of the file to open&read
@@ -105,6 +102,47 @@ stf_open_and_read (const char *filename)
 	return data;
 }
 
+static char *
+stf_preparse (IOContext *context, char const *filename)
+{
+	char *data = stf_open_and_read (filename);
+	unsigned char const *c;
+	
+	if (!data) {
+		if (context)
+			gnumeric_io_error_read (context,
+						_("Error while trying to memory map file"));
+		return NULL;
+	}
+
+	if (!stf_parse_convert_to_unix (data)) {
+		/*
+		 * Note this buffer was allocated with malloc, not g_malloc
+		 */
+		free (data);
+		if (context)
+			gnumeric_io_error_read (context,
+						_("Error while trying to pre-convert file"));
+		return NULL;
+	}
+
+	if ((c = stf_parse_is_valid_data (data)) != NULL) {
+		if (context) {
+			char *message = g_strdup_printf (_("This file does not seem to be a valid text file.\nThe character '%c' (ASCII decimal %d) was encountered.\nMost likely your locale settings are wrong."),
+							 *c, (int) *c);
+			gnumeric_io_error_read (context, message);
+			g_free (message);
+		}
+		/*
+		 * Note this buffer was allocated with malloc, not g_malloc
+		 */
+		free (data);
+		return NULL;
+	}
+
+	return data;
+}
+
 /**
  * stf_read_workbook
  * @fo       : file opener
@@ -113,57 +151,25 @@ stf_open_and_read (const char *filename)
  * @filename : file to read from+convert
  *
  * Main routine, handles importing a file including all dialog mumbo-jumbo
- *
- * returns : TRUE on success or FALSE otherwise
  **/
-static gboolean
-stf_read_workbook (FileOpener const *fo, IOContext *context, WorkbookView *wbv, char const *filename)
+static void
+stf_read_workbook (GnumFileOpener const *fo, IOContext *context, WorkbookView *wbv, char const *filename)
 {
-	Workbook *book;
 	DialogStfResult_t *dialogresult = NULL;
 	char *name;
 	char *data;
-	unsigned char *c;
 	Sheet *sheet;
+	Workbook *book;
 
-	data = stf_open_and_read (filename);
-	if (!data) {
-
-		gnumeric_io_error_read (context,
-				     _("Error while trying to memory map file"));
-		return FALSE;
-	}
-
-	if (!stf_parse_convert_to_unix (data)) {
-		/*
-		 * Note this buffer was allocated with malloc, not g_malloc
-		 */
-		free (data);
-		gnumeric_io_error_read (context,
-				     _("Error while trying to pre-convert file"));
-		return FALSE;
-	}
-
-	if ((c = stf_parse_is_valid_data (data)) != NULL) {
-		char *message;
-		/*
-		 * Note this buffer was allocated with malloc, not g_malloc
-		 */
-
-		message = g_strdup_printf (_("This file does not seem to be a valid text file.\nThe character '%c' (ASCII decimal %d) was encountered.\nMost likely your locale settings are wrong."),
-					   *c, (int) *c);
-		gnumeric_io_error_read (context, message);
-		g_free (message);
-
-		free (data);
-
-		return FALSE;
-	}
+	book = wb_view_workbook (wbv);
 
+	data = stf_preparse (context, filename);
+	if (!data)
+		return;
+		
 	/*
 	 * Add Sheet
 	 */
-	book = wb_view_workbook (wbv);
 	name = g_strdup_printf (_("Imported %s"), g_basename (filename));
 	sheet = sheet_new (book, name);
 	g_free (name);
@@ -199,6 +205,7 @@ stf_read_workbook (FileOpener const *fo, IOContext *context, WorkbookView *wbv,
 			col++;
 		}
 		
+		stf_parse_options_set_lines_to_parse (dialogresult->parseoptions, dialogresult->lines);
 		if (!stf_parse_sheet (dialogresult->parseoptions, dialogresult->newstart, sheet)) {
 
 			workbook_sheet_detach (book, sheet);
@@ -206,15 +213,13 @@ stf_read_workbook (FileOpener const *fo, IOContext *context, WorkbookView *wbv,
 			 * Note this buffer was allocated with malloc, not g_malloc
 			 */
 			free (data);
-			gnumeric_error_read (COMMAND_CONTEXT (context),
-					     _("Parse error while trying to parse data into sheet"));
-			return FALSE;
+			gnumeric_io_error_read (context, _("Parse error while trying to parse data into sheet"));
+			return;
 		}
 
 		workbook_recalc (book);
 		sheet_calc_spans (sheet, SPANCALC_RENDER);
-		workbook_set_saveinfo (book, filename, FILE_FL_MANUAL,
-		                       gnumeric_xml_get_saver ());
+		workbook_set_saveinfo (book, filename, FILE_FL_MANUAL, NULL);
 	} else
 		workbook_sheet_detach (book, sheet);
 
@@ -224,18 +229,112 @@ stf_read_workbook (FileOpener const *fo, IOContext *context, WorkbookView *wbv,
 	 */
 	free (data);
 
-	if (dialogresult != NULL) {
+	if (dialogresult != NULL)
 		stf_dialog_result_free (dialogresult);
-		return TRUE;
-	} else
-		return FALSE;
+	else
+		gnumeric_io_error_unknown (context);
 }
 
+/**
+ * stf_read_workbook_default_csv
+ * @fo       : file opener
+ * @context  : command context
+ * @book     : workbook
+ * @filename : file to read from+convert
+ *
+ * Automatic importing of a comma delimited csv file.
+ **/
+static void
+stf_read_workbook_default_csv (GnumFileOpener const *fo, IOContext *context,
+			       WorkbookView *wbv, char const *filename)
+{
+	Sheet *sheet;
+	Workbook *book;
+	char *name, *data;
+	StfParseOptions_t *po;
+
+	book = wb_view_workbook (wbv);
+	data = stf_preparse (context, filename);
+	if (!data)
+		return;
+
+	name = g_strdup_printf (_("Imported %s"), g_basename (filename));
+	sheet = sheet_new (book, name);
+	g_free (name);
+
+	workbook_sheet_attach (book, sheet, NULL);
+
+	po = stf_parse_options_new ();
+	
+	stf_parse_options_set_type (po, PARSE_TYPE_CSV);
+	stf_parse_options_set_trim_spaces (po, TRIM_TYPE_LEFT | TRIM_TYPE_RIGHT);
+	stf_parse_options_set_lines_to_parse (po, -1);
+	
+	stf_parse_options_csv_set_separators (po, ",", NULL);
+	stf_parse_options_csv_set_stringindicator (po, '"');
+	stf_parse_options_csv_set_indicator_2x_is_single (po, FALSE);
+	stf_parse_options_csv_set_duplicates (po, FALSE);
+	
+	if (!stf_parse_sheet (po, data, sheet)) {
+
+		workbook_sheet_detach (book, sheet);
+		/*
+		 * Note this buffer was allocated with malloc, not g_malloc
+		 */
+		free (data);
+		stf_parse_options_free (po);
+		gnumeric_io_error_read (context, _("Parse error while trying to parse data into sheet"));
+		return;
+	}
+	stf_parse_options_free (po);
+	
+	workbook_recalc (book);
+	sheet_calc_spans (sheet, SPANCALC_RENDER);
+	workbook_set_saveinfo (book, filename, FILE_FL_MANUAL, NULL);
+	
+	/*
+	 * Note the buffer was allocated with malloc, not with g_malloc
+	 * as g_malloc aborts if there is not enough memory
+	 */
+	free (data);
+}
+
+#define STF_PROBE_SIZE 16384
+
+static gboolean
+stf_read_default_probe (GnumFileOpener const *fo, const gchar *file_name, FileProbeLevel pl)
+{
+	int fd = open (file_name, O_RDONLY);
+	gboolean res;
+	char *data;
+
+	if (fd < 0)
+		return FALSE;
+
+	data = g_new (char, STF_PROBE_SIZE);
+	if (!data) {
+		close (fd);
+		return FALSE;
+	}
+		
+	if (read (fd, data, STF_PROBE_SIZE) < 1) {
+		g_free (data);
+		close (fd);
+		return FALSE;
+	}
+
+	res = (stf_parse_is_valid_data (data) == NULL);
+	g_free (data);
+	close (fd);
+	return res;
+}
 
 #ifndef PAGE_SIZE
 #define PAGE_SIZE (BUFSIZ*8)
 #endif
 
+/***********************************************************************************/
+
 /**
  * stf_open_for_write:
  * @context: commandcontext
@@ -284,45 +383,43 @@ stf_write_func (const char *string, FILE *f)
  * @filename : file to read from+convert
  *
  * Main routine, handles exporting a file including all dialog mumbo-jumbo
- *
- * returns : TRUE on success or FALSE otherwise
  **/
-static gboolean
-stf_write_workbook (FileSaver const *fs, IOContext *context, WorkbookView *wb_view, const char *filename)
+static void
+stf_write_workbook (GnumFileSaver const *fs, IOContext *context, WorkbookView *wbv, const char *filename)
 {
 	StfE_Result_t *result = NULL;
+	
 
-	g_return_val_if_fail (context != NULL, FALSE);
-	g_return_val_if_fail (wb_view != NULL, FALSE);
-	g_return_val_if_fail (filename != NULL, FALSE);
+	g_return_if_fail (context != NULL);
+	g_return_if_fail (wbv != NULL);
+	g_return_if_fail (filename != NULL);
 
 	if (IS_WORKBOOK_CONTROL_GUI (context->impl))
 		result = stf_export_dialog (WORKBOOK_CONTROL_GUI (context->impl),
-					    wb_view_workbook (wb_view));
+		         wb_view_workbook (wbv));
 
 	if (result != NULL) {
 		FILE *f = stf_open_for_write (context, filename);
 
-		if (!f)
-			return FALSE;
+		if (f == NULL) {
+			gnumeric_io_error_unknown (context);
+			return;
+		}
 
 		stf_export_options_set_write_callback (result->export_options,
 						       (StfEWriteFunc) stf_write_func, (gpointer) f);
-		fclose (f);
-
 		if (stf_export (result->export_options) == FALSE) {
-
-			gnumeric_error_read (COMMAND_CONTEXT (context),
-					     _("Error while trying to write csv file"));
+			gnumeric_io_error_read (context,
+			_("Error while trying to write csv file"));
 			stf_export_dialog_result_free (result);
-			return FALSE;
+			return;
 		}
 
+		fclose (f);
 		stf_export_dialog_result_free (result);
-
-		return TRUE;
-	} else
-		return FALSE;
+	} else {
+		gnumeric_io_error_unknown (context);
+	}
 }
 /**
  * stf_init
@@ -334,12 +431,15 @@ stf_write_workbook (FileSaver const *fs, IOContext *context, WorkbookView *wb_vi
 void
 stf_init (void)
 {
-	const char *desc;
-
-	desc = _("Text File import");
-	file_format_register_open (1, desc, NULL, stf_read_workbook);
-
-	desc = _("Text File Export (*.csv)");
-	file_format_register_save ("csv", desc, FILE_FL_MANUAL,
-	                           stf_write_workbook);
+	register_file_opener_as_importer (gnum_file_opener_new (
+	                      "Gnumeric_stf:stf_druid", _("Text File import (customizable)"),
+	                      NULL, stf_read_workbook));
+	register_file_opener (gnum_file_opener_new (
+		"Gnumeric_stf:stf_csv", _("Text File import (default csv)"),
+		stf_read_default_probe, stf_read_workbook_default_csv), 0);
+			      
+	register_file_saver (gnum_file_saver_new (
+	                     "Gnumeric_stf:stf", "csv",
+	                     _("Text File Export (*.csv)"),
+	                     FILE_FL_MANUAL, stf_write_workbook));
 }
