@@ -1,3 +1,4 @@
+/* vim: set sw=8: */
 /*
  * xml-io.c: save/read gnumeric Sheets using an XML encoding.
  *
@@ -7,10 +8,7 @@
  *   Jody Goldberg <jgoldberg@home.com>
  */
 #include <config.h>
-#include "gnumeric.h"
-#include "gnome-xml/parser.h"
-#include "gnome-xml/parserInternals.h"
-#include "gnome-xml/xmlmemory.h"
+#include "xml-io.h"
 #include "style-color.h"
 #include "style-border.h"
 #include "style.h"
@@ -19,7 +17,6 @@
 #include "sheet-object.h"
 #include "sheet-object-cell-comment.h"
 #include "print-info.h"
-#include "xml-io.h"
 #include "file.h"
 #include "expr.h"
 #include "expr-name.h"
@@ -37,26 +34,42 @@
 #include "ranges.h"
 #include "file.h"
 #include "str.h"
+#include "plugin-util.h"
 
+#include <gnome-xml/parser.h>
+#include <gnome-xml/parserInternals.h>
+#include <gnome-xml/xmlmemory.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <zlib.h>
 #include <gal/util/e-xml-utils.h>
 #include <gnome.h>
 #include <locale.h>
 #include <math.h>
 #include <limits.h>
+#ifdef ENABLE_BONOBO
+#include <bonobo/bonobo-exception.h>
+#endif
 
 /* Precision to use when saving point measures. */
 #define POINT_SIZE_PRECISION 3
 
-static FileOpener *xml_opener = NULL;
-static FileSaver  *xml_saver = NULL;
+/* FIXME - tune the values below */
+#define XML_INPUT_BUFFER_SIZE      4096
+#define N_ELEMENTS_BETWEEN_UPDATES 20
 
-FileOpener *
+static GnumFileOpener *xml_opener = NULL;
+static GnumFileSaver  *xml_saver = NULL;
+
+GnumFileOpener *
 gnumeric_xml_get_opener (void)
 {
 	return xml_opener;
 }
 
-FileSaver *
+GnumFileSaver *
 gnumeric_xml_get_saver (void)
 {
 	return xml_saver;
@@ -255,6 +268,27 @@ xml_get_value_int (xmlNodePtr node, const char *name, int *val)
 	return FALSE;
 }
 
+static gboolean
+xml_get_prop_cellpos (xmlNodePtr node, char const *name, CellPos *val)
+{
+	char *str;
+	int dummy;
+	gboolean res;
+
+	str = (char *) xmlGetProp (node, name);
+	if (val == NULL)
+		return FALSE;
+	res = parse_cell_name (str, &val->col, &val->row, TRUE, &dummy);
+	xmlFree (str);
+	return res;
+}
+
+static void
+xml_set_prop_cellpos (xmlNodePtr node, char const *name, CellPos const *val)
+{
+	xmlSetProp (node, name, cell_pos_name (val));
+}
+
 #if 0
 /*
  * Get a float value for a node either carried as an attibute or as
@@ -325,33 +359,6 @@ xml_set_value_cstr (xmlNodePtr node, const char *name, const char *val)
 	xmlSetProp (node, name, val);
 }
 
-/*
- * Set a String value for a node either carried as an attibute or as
- * the content of a child.
- */
-void
-xml_set_value_string (xmlNodePtr node, const char *name, const String *val)
-{
-	char *ret;
-	xmlNodePtr child;
-
-	ret = xmlGetProp (node, name);
-	if (ret != NULL) {
-		xmlFree (ret);
-		xmlSetProp (node, name, val->str);
-		return;
-	}
-	child = node->xmlChildrenNode;
-	while (child != NULL){
-		if (!strcmp (child->name, name)){
-			xmlNodeSetContent (child, val->str);
-			return;
-		}
-		child = child->next;
-	}
-	xmlSetProp (node, name, val->str);
-}
-
 /*
  * Set an integer value for a node either carried as an attibute or as
  * the content of a child.
@@ -386,11 +393,9 @@ xml_set_value_int (xmlNodePtr node, const char *name, int val)
  * the content of a child.
  */
 static void
-xml_set_value_double (xmlNodePtr node, const char *name, double val,
-		      int precision)
+xml_set_prop_double (xmlNodePtr node, const char *name, double val,
+		     int precision)
 {
-	char *ret;
-	xmlNodePtr child;
 	char str[101 + DBL_DIG];
 
 	if (precision < 0 || precision > DBL_DIG)
@@ -401,20 +406,6 @@ xml_set_value_double (xmlNodePtr node, const char *name, double val,
 	else
 		snprintf (str, 100 + DBL_DIG, "%f", val);
 
-	ret = xmlGetProp (node, name);
-	if (ret != NULL){
-		xmlFree (ret);
-		xmlSetProp (node, name, str);
-		return;
-	}
-	child = node->xmlChildrenNode;
-	while (child != NULL){
-		if (!strcmp (child->name, name)){
-			xmlNodeSetContent (child, str);
-			return;
-		}
-		child = child->next;
-	}
 	xmlSetProp (node, name, str);
 }
 
@@ -424,7 +415,7 @@ xml_set_value_double (xmlNodePtr node, const char *name, double val,
 static void
 xml_set_value_points (xmlNodePtr node, const char *name, double val)
 {
-	xml_set_value_double (node, name, val, POINT_SIZE_PRECISION);
+	xml_set_prop_double (node, name, val, POINT_SIZE_PRECISION);
 }
 
 static void
@@ -501,20 +492,24 @@ xml_read_range (xmlNodePtr tree, Range *r)
 }
 
 static void
-xml_write_range (xmlNodePtr tree, const Range *value)
+xml_write_range (xmlNodePtr tree, Range const *r)
 {
-	xml_set_value_int (tree, "startCol", value->start.col);
-	xml_set_value_int (tree, "startRow", value->start.row);
-	xml_set_value_int (tree, "endCol",   value->end.col);
-	xml_set_value_int (tree, "endRow",   value->end.row);
+	g_return_if_fail (range_is_sane (r));
+
+	xml_set_value_int (tree, "startCol", r->start.col);
+	xml_set_value_int (tree, "startRow", r->start.row);
+	xml_set_value_int (tree, "endCol",   r->end.col);
+	xml_set_value_int (tree, "endRow",   r->end.row);
 }
 
 static void
-xml_read_selection_info (XmlParseContext *ctxt, Sheet *sheet, xmlNodePtr tree)
+xml_read_selection_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	Range r;
 	int row, col;
+	Sheet *sheet = ctxt->sheet;
 	xmlNodePtr sel, selections = e_xml_get_child_by_name (tree, "Selections");
+
 	if (selections == NULL)
 		return;
 
@@ -1272,7 +1267,7 @@ xml_read_print_margins (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	g_return_if_fail (ctxt != NULL);
 	g_return_if_fail (tree != NULL);
-	g_return_if_fail (ctxt->sheet != NULL);
+	g_return_if_fail (IS_SHEET (ctxt->sheet));
 
 	pi = ctxt->sheet->print_info;
 
@@ -1328,7 +1323,7 @@ xml_read_print_info (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	g_return_if_fail (ctxt != NULL);
 	g_return_if_fail (tree != NULL);
-	g_return_if_fail (ctxt->sheet != NULL);
+	g_return_if_fail (IS_SHEET (ctxt->sheet));
 
 	pi = ctxt->sheet->print_info;
 
@@ -1439,11 +1434,6 @@ style_font_read_from_x11 (MStyle *mstyle, const char *fontname)
 {
 	const char *c;
 
-	/*
-	 * FIXME: we should do something about the typeface instead
-	 * of hardcoding it to helvetica.
-	 */
-
 	c = font_component (fontname, 2);
 	if (strncmp (c, "bold", 4) == 0)
 		mstyle_set_font_bold (mstyle, TRUE);
@@ -1787,11 +1777,9 @@ xml_cell_set_array_expr (Cell *cell, char const *text,
 
 	g_return_if_fail (expr != NULL);
 	cell_set_array_formula (cell->base.sheet,
-				cell->pos.row,
-				cell->pos.col,
-				cell->pos.row + rows-1,
-				cell->pos.col + cols-1,
-				expr, TRUE);
+				cell->pos.col, cell->pos.row,
+				cell->pos.col + cols-1, cell->pos.row + rows-1,
+				expr);
 }
 
 /**
@@ -1823,13 +1811,13 @@ xml_not_used_old_array_spec (Cell *cell, char *content)
 	rows = strtol (ptr = expr_end + 2, &end, 10);
 	if (end == ptr || *end != ',')
 		return TRUE;
-	cols = strtol (ptr = ++end, &end, 10);
+	cols = strtol (ptr = end + 1, &end, 10);
 	if (end == ptr || end[0] != ')' || end[1] != '[')
 		return TRUE;
-	row = strtol (ptr = (end += 2), &end, 10);
+	row = strtol (ptr = (end + 2), &end, 10);
 	if (end == ptr || end[0] != ']' || end[1] != '[')
 		return TRUE;
-	col = strtol (ptr = (end += 2), &end, 10);
+	col = strtol (ptr = (end + 2), &end, 10);
 	if (end == ptr || end[0] != ']' || end[1] != '\0')
 		return TRUE;
 
@@ -1985,7 +1973,7 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 		if (shared_expr_index > 0) {
 			gpointer id = GINT_TO_POINTER (shared_expr_index);
 			gpointer expr =
-			    g_hash_table_lookup (ctxt->expr_map, id);
+				g_hash_table_lookup (ctxt->expr_map, id);
 
 			if (expr == NULL) {
 				if (cell_has_expr (ret))
@@ -2090,6 +2078,40 @@ xml_write_merged_regions (XmlParseContext const *ctxt,
 	}
 }
 
+static void
+xml_read_sheet_layout (XmlParseContext *ctxt, xmlNodePtr tree)
+{
+	xmlNodePtr child;
+	CellPos tmp, frozen_tl, unfrozen_tl;
+
+	tree = e_xml_get_child_by_name (tree, "SheetLayout");
+	if (tree == NULL)
+	    return;
+
+	/* The top left cell in pane[0] */
+	if (xml_get_prop_cellpos (tree, "TopLeft", &tmp))
+		sheet_set_initial_top_left (ctxt->sheet, tmp.col, tmp.row);
+
+	child = e_xml_get_child_by_name (tree, "FreezePanes");
+	if (child != NULL &&
+	    xml_get_prop_cellpos (child, "FrozenTopLeft", &frozen_tl) &&
+	    xml_get_prop_cellpos (child, "UnfrozenTopLeft", &unfrozen_tl))
+		sheet_freeze_panes (ctxt->sheet, &frozen_tl, &unfrozen_tl);
+}
+
+static void
+xml_write_sheet_layout (XmlParseContext *ctxt, xmlNodePtr tree, Sheet const *sheet)
+{
+	tree = xmlNewChild (tree, ctxt->ns, "SheetLayout", NULL);
+
+	xml_set_prop_cellpos (tree, "TopLeft", &sheet->initial_top_left);
+	if (sheet_is_frozen (sheet)) {
+		xmlNodePtr freeze = xmlNewChild (tree, ctxt->ns, "FreezePanes", NULL);
+		xml_set_prop_cellpos (freeze, "FrozenTopLeft", &sheet->frozen_top_left);
+		xml_set_prop_cellpos (freeze, "UnfrozenTopLeft", &sheet->unfrozen_top_left);
+	}
+}
+
 static xmlNodePtr
 xml_write_styles (XmlParseContext *ctxt, StyleList *styles)
 {
@@ -2111,13 +2133,18 @@ xml_write_styles (XmlParseContext *ctxt, StyleList *styles)
 }
 
 static void
-xml_read_solver (Sheet *sheet, XmlParseContext *ctxt, xmlNodePtr tree,
-		 SolverParameters *param)
+xml_read_solver (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	SolverConstraint *c;
 	xmlNodePtr       child;
 	int              col, row;
 	String           *s;
+	Sheet *sheet = ctxt->sheet;
+	SolverParameters *param = &sheet->solver_parameters;
+
+	tree = e_xml_get_child_by_name (tree, "Solver");
+	if (tree == NULL)
+		return;
 
 	xml_get_value_int (tree, "TargetCol", &col);
 	xml_get_value_int (tree, "TargetRow", &row);
@@ -2197,7 +2224,7 @@ xml_write_solver (XmlParseContext *ctxt, SolverParameters const *param)
 	xml_set_value_int (cur, "ProblemType", param->problem_type);
 
 	s = string_get (param->input_entry_str);
-	xml_set_value_string (cur, "Inputs", s);
+	xmlSetProp (cur, "Inputs", s->str);
 	string_unref (s);
 
 	constraints = param->constraints;
@@ -2415,10 +2442,8 @@ xml_sheet_write (XmlParseContext *ctxt, Sheet const *sheet)
 	}
 
 	xml_write_merged_regions (ctxt, cur, sheet->list_merged);
+	xml_write_sheet_layout (ctxt, cur, sheet);
 
-	/*
-	 * Solver informations
-	 */
 	solver = xml_write_solver (ctxt, &sheet->solver_parameters);
 	if (solver)
 		xmlAddChild (cur, solver);
@@ -2528,8 +2553,10 @@ xml_read_styles (XmlParseContext *ctxt, xmlNodePtr tree)
 	if (child == NULL)
 		return;
 
-	for (regions = child->xmlChildrenNode; regions; regions = regions->next)
+	for (regions = child->xmlChildrenNode; regions != NULL; regions = regions->next) {
 		xml_read_style_region (ctxt, regions);
+		count_io_progress_update (ctxt->io_context, 1);
+	}
 }
 
 /*
@@ -2584,10 +2611,11 @@ xml_read_colrow_info (XmlParseContext *ctxt, xmlNodePtr tree,
 }
 
 static void
-xml_read_cols_info (XmlParseContext *ctxt, Sheet *sheet, xmlNodePtr tree)
+xml_read_cols_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr cols, col;
 	double tmp;
+	Sheet *sheet = ctxt->sheet;
 
 	cols = e_xml_get_child_by_name (tree, "Cols");
 	if (cols == NULL)
@@ -2613,10 +2641,11 @@ xml_read_cols_info (XmlParseContext *ctxt, Sheet *sheet, xmlNodePtr tree)
 }
 
 static void
-xml_read_rows_info (XmlParseContext *ctxt, Sheet *sheet, xmlNodePtr tree)
+xml_read_rows_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr rows, row;
 	double tmp;
+	Sheet *sheet = ctxt->sheet;
 
 	rows = e_xml_get_child_by_name (tree, "Rows");
 	if (rows == NULL)
@@ -2686,7 +2715,7 @@ xml_sheet_read (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr child;
 	/* xmlNodePtr styles; */
-	Sheet *ret = NULL;
+	Sheet *sheet = NULL;
 	double zoom_factor;
 	char *val;
 
@@ -2704,49 +2733,49 @@ xml_sheet_read (XmlParseContext *ctxt, xmlNodePtr tree)
 	 */
 	val = xml_value_get (tree, "Name");
 	if (val != NULL){
-		ret = workbook_sheet_by_name (ctxt->wb, (const char *) val);
-		if (ret == NULL)
-			ret = sheet_new (ctxt->wb, (const char *) val);
+		sheet = workbook_sheet_by_name (ctxt->wb, (const char *) val);
+		if (sheet == NULL)
+			sheet = sheet_new (ctxt->wb, (const char *) val);
 		g_free (val);
 	}
 
-	if (ret == NULL)
+	if (sheet == NULL)
 		return NULL;
 
-	ctxt->sheet = ret;
+	ctxt->sheet = sheet;
 
-	ret->display_formulas = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->display_formulas = e_xml_get_bool_prop_by_name_with_default (tree,
 		"DisplayFormulas", FALSE);
-	ret->hide_zero = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->hide_zero = e_xml_get_bool_prop_by_name_with_default (tree,
 		"HideZero", FALSE);
-	ret->hide_grid = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->hide_grid = e_xml_get_bool_prop_by_name_with_default (tree,
 		"HideGrid", FALSE);
-	ret->hide_col_header = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->hide_col_header = e_xml_get_bool_prop_by_name_with_default (tree,
 		"HideColHeader", FALSE);
-	ret->hide_row_header = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->hide_row_header = e_xml_get_bool_prop_by_name_with_default (tree,
 		"HideRowHeader", FALSE);
-	ret->display_outlines = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->display_outlines = e_xml_get_bool_prop_by_name_with_default (tree,
 		"DisplayOutlines", TRUE);
-	ret->outline_symbols_below = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->outline_symbols_below = e_xml_get_bool_prop_by_name_with_default (tree,
 		"OutlineSymbolsBelow", TRUE);
-	ret->outline_symbols_right = e_xml_get_bool_prop_by_name_with_default (tree,
+	sheet->outline_symbols_right = e_xml_get_bool_prop_by_name_with_default (tree,
 		"OutlineSymbolsRight", TRUE);
 
-	xml_get_value_int (tree, "MaxCol", &ret->cols.max_used);
-	xml_get_value_int (tree, "MaxRow", &ret->rows.max_used);
+	xml_get_value_int (tree, "MaxCol", &sheet->cols.max_used);
+	xml_get_value_int (tree, "MaxRow", &sheet->rows.max_used);
 	xml_get_value_double (tree, "Zoom", &zoom_factor);
 
 	xml_read_print_info (ctxt, tree);
 	xml_read_styles (ctxt, tree);
 	xml_read_cell_styles (ctxt, tree);
-	xml_read_cols_info (ctxt, ret, tree);
-	xml_read_rows_info (ctxt, ret, tree);
+	xml_read_cols_info (ctxt, tree);
+	xml_read_rows_info (ctxt, tree);
 	xml_read_merged_regions (ctxt, tree);
-	xml_read_selection_info (ctxt, ret, tree);
+	xml_read_selection_info (ctxt, tree);
 
 	child = e_xml_get_child_by_name (tree, "Names");
 	if (child)
-		xml_read_names (ctxt, child, NULL, ret);
+		xml_read_names (ctxt, child, NULL, sheet);
 
 	child = e_xml_get_child_by_name (tree, "Objects");
 	if (child != NULL) {
@@ -2757,22 +2786,24 @@ xml_sheet_read (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	child = e_xml_get_child_by_name (tree, "Cells");
 	if (child != NULL) {
-		xmlNodePtr cell = child->xmlChildrenNode;
-		for (; cell != NULL ; cell = cell->next)
+		xmlNodePtr cell;
+
+		for (cell = child->xmlChildrenNode; cell != NULL ; cell = cell->next) {
 			xml_read_cell (ctxt, cell);
+			count_io_progress_update (ctxt->io_context, 1);
+		}
 	}
 
-	child = e_xml_get_child_by_name (tree, "Solver");
-	if (child != NULL)
-	        xml_read_solver (ret, ctxt, child, &(ret->solver_parameters));
+	xml_read_solver (ctxt, tree);
+	xml_read_sheet_layout (ctxt, tree);
 
 	xml_dispose_read_cell_styles (ctxt);
 
 	/* Init ColRowInfo's size_pixels and force a full respan */
-	sheet_flag_recompute_spans (ret);
-	sheet_set_zoom_factor (ret, zoom_factor, FALSE, FALSE);
+	sheet_flag_recompute_spans (sheet);
+	sheet_set_zoom_factor (sheet, zoom_factor, FALSE, FALSE);
 
-	return ret;
+	return sheet;
 }
 
 /*
@@ -2827,6 +2858,41 @@ xml_read_selection_clipboard (XmlParseContext *ctxt, xmlNodePtr tree)
 	return cr;
 }
 
+/* These will be searched IN ORDER, so add new versions at the top */
+static const struct {
+	char const * const id;
+	GnumericXMLVersion const version;
+} GnumericVersions [] = {
+	{ "http://www.gnome.org/gnumeric/v7", GNUM_XML_V7 },	/* 0.66 */
+	{ "http://www.gnome.org/gnumeric/v6", GNUM_XML_V6 },	/* 0.62 */
+	{ "http://www.gnome.org/gnumeric/v5", GNUM_XML_V5 },
+	{ "http://www.gnome.org/gnumeric/v4", GNUM_XML_V4 },
+	{ "http://www.gnome.org/gnumeric/v3", GNUM_XML_V3 },
+	{ "http://www.gnome.org/gnumeric/v2", GNUM_XML_V2 },
+	{ "http://www.gnome.org/gnumeric/", GNUM_XML_V1 },
+	{ NULL }
+};
+
+xmlNsPtr
+xml_check_version (xmlDocPtr doc, GnumericXMLVersion *version)
+{
+	xmlNsPtr gmr;
+	int i;
+
+	/* Do a bit of checking, get the namespaces, and check the top elem.  */
+	if (doc->xmlRootNode->name == NULL || strcmp (doc->xmlRootNode->name, "Workbook"))
+		return NULL;
+
+	for (i = 0 ; GnumericVersions [i].id != NULL ; ++i ) {
+		gmr = xmlSearchNsByHref (doc, doc->xmlRootNode, GnumericVersions [i].id);
+		if (gmr != NULL) {
+			*version = GnumericVersions [i].version;
+			return gmr;
+		}
+	}
+	return NULL;
+}
+
 /*
  * Create an XML subtree of doc equivalent to the given Workbook.
  */
@@ -2849,7 +2915,11 @@ xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
 	if (cur == NULL)
 		return NULL;
 	if (ctxt->ns == NULL) {
-		gmr = xmlNewNs (cur, "http://www.gnome.org/gnumeric/v6", "gmr");
+		/* GnumericVersions[0] is always the first item and
+		 * the most recent version, see table above. Keep the table
+		 * ordered this way!
+		 */
+		gmr = xmlNewNs (cur, GnumericVersions[0].id, "gmr");
 		xmlSetNs(cur, gmr);
 		ctxt->ns = gmr;
 	}
@@ -2871,10 +2941,30 @@ xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
 	if (child)
 		xmlAddChild (cur, child);
 
+	/* The sheet name index is required for the xml_sax
+	 * importer to work correctly. We don't use it for
+	 * the dom loader! These must be written BEFORE
+	 * the named expressions.
+	 */
+	child = xmlNewChild (cur, ctxt->ns, "SheetNameIndex", NULL);
+	sheets0 = sheets = workbook_sheets (wb);
+	while (sheets) {
+		char *tstr;
+		Sheet *sheet = sheets->data;
+
+		tstr = xmlEncodeEntitiesReentrant (ctxt->doc, sheet->name_unquoted);
+		xmlNewChild (child, ctxt->ns, "SheetName",  tstr);
+		if (tstr)
+			xmlFree (tstr);
+			
+		sheets = g_list_next (sheets);
+	}
+	xmlAddChild (cur, child);
+	
 	child = xml_write_names (ctxt, wb->names);
 	if (child)
 		xmlAddChild (cur, child);
-
+	
 /*	child = xml_write_style (ctxt, &wb->style, -1);
 	if (child)
 	xmlAddChild (cur, child);*/
@@ -2890,7 +2980,7 @@ xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
 	child = xmlNewChild (cur, ctxt->ns, "Sheets", NULL);
 	ctxt->parent = child;
 
-	sheets0 = sheets = workbook_sheets (wb);
+	sheets = sheets0;
 	while (sheets) {
 		xmlNodePtr cur, parent;
 		Sheet *sheet = sheets->data;
@@ -2942,6 +3032,51 @@ xml_sheet_create (XmlParseContext *ctxt, xmlNodePtr node)
 	}
 }
 
+static gint
+xml_get_n_children (xmlNodePtr tree)
+{
+	gint n = 0;
+	xmlNodePtr node;
+
+	for (node = tree->xmlChildrenNode; node != NULL; node = node->next)
+		n++;
+
+	return n;
+}
+
+static gint 
+xml_read_sheet_n_elements (xmlNodePtr tree)
+{
+	gint n = 0;
+	xmlNodePtr node;
+
+	node = e_xml_get_child_by_name (tree, "Styles");
+	if (node != NULL) {
+		n += xml_get_n_children (node);
+	}
+	node = e_xml_get_child_by_name (tree, "Cells");
+	if (node != NULL) {
+		n += xml_get_n_children (node);
+	}
+
+	return n;
+}
+
+static gint
+xml_read_workbook_n_elements (xmlNodePtr tree)
+{
+	xmlNodePtr node;
+	gint n = 0;
+
+	for (node = tree->xmlChildrenNode; node != NULL; node = node->next) {
+		if (node->name != NULL && strcmp (node->name, "Sheet") == 0) {
+			n += xml_read_sheet_n_elements (node);
+		}
+	}
+
+	return n;
+}
+
 /*
  * Create a Workbook equivalent to the XML subtree of doc.
  */
@@ -2996,11 +3131,8 @@ xml_workbook_read (IOContext *context, WorkbookView *wb_view,
 	 * all of the references to forward sheets are properly
 	 * handled
 	 */
-	c = child->xmlChildrenNode;
-	while (c != NULL){
+	for (c = child->xmlChildrenNode; c != NULL ; c = c->next)
 		xml_sheet_create (ctxt, c);
-		c = c->next;
-	}
 
 	/*
 	 * Now read names which can have inter-sheet references
@@ -3011,19 +3143,27 @@ xml_workbook_read (IOContext *context, WorkbookView *wb_view,
 		xml_read_names (ctxt, child, wb, NULL);
 
 	child = e_xml_get_child_by_name (tree, "Sheets");
+
 	/*
 	 * Pass 2: read the contents
 	 */
+	io_progress_message (context, _("Processing XML tree..."));
+	io_progress_range_push (context, 0.5, 1.0);
+	count_io_progress_set (context, xml_read_workbook_n_elements (child),
+	                       N_ELEMENTS_BETWEEN_UPDATES);
+	ctxt->io_context = context;
 	c = child->xmlChildrenNode;
 	while (c != NULL) {
 		sheet = xml_sheet_read (ctxt, c);
 		c = c->next;
 	}
+	io_progress_unset (context);
+	io_progress_range_pop (context);
 
 	child = e_xml_get_child_by_name (tree, "Attributes");
 	if (child && ctxt->version >= GNUM_XML_V5) {
 		xml_read_attributes (ctxt, child, &list);
-		wb_view_set_attributev (wb_view, list);
+		wb_view_set_attribute_list (wb_view, list);
 		xml_free_arg_list (list);
 		g_list_free (list);
 	}
@@ -3043,57 +3183,29 @@ xml_workbook_read (IOContext *context, WorkbookView *wb_view,
 	gnumeric_setlocale (LC_NUMERIC, old_num_locale);
 	g_free (old_num_locale);
 
+	workbook_queue_all_recalc (wb);
+
 	return TRUE;
 }
 
-/* These will be searched IN ORDER, so add new versions at the top */
-static const struct {
-	char const * const id;
-	GnumericXMLVersion const version;
-} GnumericVersions [] = {
-	{ "http://www.gnome.org/gnumeric/v6", GNUM_XML_V6 },	/* 0.62 */
-	{ "http://www.gnome.org/gnumeric/v5", GNUM_XML_V5 },
-	{ "http://www.gnome.org/gnumeric/v4", GNUM_XML_V4 },
-	{ "http://www.gnome.org/gnumeric/v3", GNUM_XML_V3 },
-	{ "http://www.gnome.org/gnumeric/v2", GNUM_XML_V2 },
-	{ "http://www.gnome.org/gnumeric/", GNUM_XML_V1 },
-	{ NULL }
-};
-
-xmlNsPtr
-xml_check_version (xmlDocPtr doc, GnumericXMLVersion *version)
-{
-	xmlNsPtr gmr;
-	int i;
-
-	/* Do a bit of checking, get the namespaces, and check the top elem.  */
-	if (doc->xmlRootNode->name == NULL || strcmp (doc->xmlRootNode->name, "Workbook"))
-		return NULL;
-
-	for (i = 0 ; GnumericVersions [i].id != NULL ; ++i ) {
-		gmr = xmlSearchNsByHref (doc, doc->xmlRootNode, GnumericVersions [i].id);
-		if (gmr != NULL) {
-			*version = GnumericVersions [i].version;
-			return gmr;
-		}
-	}
-	return NULL;
-}
-
 /*
  * We parse and do some limited validation of the XML file, if this
  * passes, then we return TRUE
  */
 static gboolean
-xml_probe (FileOpener const *fo, const gchar *filename)
+xml_probe (GnumFileOpener const *fo, const gchar *filename, FileProbeLevel pl)
 {
-        int ret;
+	int ret;
 	xmlDocPtr res = NULL;
 	xmlNsPtr gmr;
 	xmlParserCtxtPtr ctxt;
 	xmlSAXHandler silent, *old;
 	GnumericXMLVersion version;
 
+	if (pl == FILE_PROBE_FILE_NAME) {
+		return strcmp (g_extension_pointer (filename), "gnumeric") == 0;
+	}
+
 	/*
 	 * Do a silent call to the XML parser.
 	 */
@@ -3143,7 +3255,7 @@ gnumeric_xml_write_selection_clipboard (WorkbookControl *wbc, Sheet *sheet,
 	xmlDocPtr xml;
 	XmlParseContext ctxt;
 
-	g_return_val_if_fail (sheet != NULL, -1);
+	g_return_val_if_fail (IS_SHEET (sheet), -1);
 
 	/*
 	 * Create the tree
@@ -3218,32 +3330,77 @@ gnumeric_xml_read_selection_clipboard (WorkbookControl *wbc, CellRegion **cr,
  * One parse the XML file, getting a tree, then analyze the tree to build
  * the actual in-memory structure.
  */
-gboolean
-gnumeric_xml_read_workbook (FileOpener const *fo,
+void
+gnumeric_xml_read_workbook (GnumFileOpener const *fo,
                             IOContext *context,
                             WorkbookView *wbv,
                             const gchar *filename)
 {
+	gint fd;
+	gzFile *f;
+	gint file_size;
+	ErrorInfo *open_error;
+	gchar buffer[XML_INPUT_BUFFER_SIZE];
+	gint bytes;
+	xmlParserCtxtPtr pctxt;
 	xmlDocPtr res;
 	xmlNsPtr gmr;
 	XmlParseContext *ctxt;
 	GnumericXMLVersion    version;
 
-	g_return_val_if_fail (filename != NULL, FALSE);
+	g_return_if_fail (filename != NULL);
 
 	/*
 	 * Load the file into an XML tree.
 	 */
-	res = xmlParseFile (filename);
+	fd = gnumeric_open_error_info (filename, O_RDONLY, &open_error);
+	if (fd < 0) {
+		gnumeric_io_error_info_set (context, open_error);
+		return;
+	}
+	file_size = lseek (fd, 0, SEEK_END);
+	if (file_size < 0 || lseek (fd, 0, SEEK_SET) < 0) {
+		if (errno == 0) {
+			gnumeric_io_error_info_set (context, error_info_new_str (
+			_("Not enough memory to create zlib decompression state.")));
+		} else {
+			gnumeric_io_error_info_set (context, error_info_new_from_errno ());
+		}
+		close (fd);
+		return;
+	}
+	f = gzdopen (fd, "r");
+	if (f == NULL) {
+		close (fd);
+		gnumeric_io_error_info_set (context, error_info_new_str (
+		_("Not enough memory to create zlib decompression state.")));
+		return;
+	}
+	io_progress_message (context, _("Parsing XML file..."));
+	io_progress_range_push (context, 0.0, 0.5);
+	value_io_progress_set (context, file_size, 0);
+	bytes = gzread (f, buffer, 4);
+	pctxt = xmlCreatePushParserCtxt (NULL, NULL, buffer, bytes, filename);
+	while ((bytes = gzread (f, buffer, XML_INPUT_BUFFER_SIZE)) > 0) {
+		xmlParseChunk (pctxt, buffer, bytes, 0);
+		value_io_progress_update (context, lseek (fd, 0, SEEK_CUR));
+	}
+	xmlParseChunk (pctxt, buffer, 0, 1);
+	res = pctxt->myDoc;
+	xmlFreeParserCtxt (pctxt);
+	gzclose (f);
+	io_progress_unset (context);
+	io_progress_range_pop (context);
+
 	if (res == NULL) {
 		gnumeric_io_error_read (context, "");
-		return FALSE;
+		return;
 	}
 	if (res->xmlRootNode == NULL) {
 		xmlFreeDoc (res);
 		gnumeric_io_error_read (context,
 			_("Invalid xml file. Tree is empty ?"));
-		return FALSE;
+		return;
 	}
 
 	/* Do a bit of checking, get the namespaces, and check the top elem. */
@@ -3252,7 +3409,7 @@ gnumeric_xml_read_workbook (FileOpener const *fo,
 		xmlFreeDoc (res);
 		gnumeric_io_error_read (context,
 			_("Is not an Gnumeric Workbook file"));
-		return FALSE;
+		return;
 	}
 
 	/* Parse the file */
@@ -3263,8 +3420,6 @@ gnumeric_xml_read_workbook (FileOpener const *fo,
 	                       gnumeric_xml_get_saver ());
 	xml_parse_ctx_destroy (ctxt);
 	xmlFreeDoc (res);
-
-	return TRUE;
 }
 
 /*
@@ -3272,8 +3427,8 @@ gnumeric_xml_read_workbook (FileOpener const *fo,
  * One build an in-memory XML tree and save it to a file.
  * returns 0 in case of success, -1 otherwise.
  */
-gboolean
-gnumeric_xml_write_workbook (FileSaver const *fs,
+void
+gnumeric_xml_write_workbook (GnumFileSaver const *fs,
                              IOContext *context,
                              WorkbookView *wb_view,
                              const gchar *filename)
@@ -3282,8 +3437,8 @@ gnumeric_xml_write_workbook (FileSaver const *fs,
 	XmlParseContext *ctxt;
 	int ret;
 
-	g_return_val_if_fail (wb_view != NULL, FALSE);
-	g_return_val_if_fail (filename != NULL, FALSE);
+	g_return_if_fail (wb_view != NULL);
+	g_return_if_fail (filename != NULL);
 
 	/*
 	 * Create the tree
@@ -3291,7 +3446,7 @@ gnumeric_xml_write_workbook (FileSaver const *fs,
 	xml = xmlNewDoc ("1.0");
 	if (xml == NULL) {
 		gnumeric_io_error_save (context, "");
-		return FALSE;
+		return;
 	}
 	ctxt = xml_parse_ctx_new (xml, NULL);
 	xml->xmlRootNode = xml_workbook_write (ctxt, wb_view);
@@ -3305,10 +3460,7 @@ gnumeric_xml_write_workbook (FileSaver const *fs,
 	xmlFreeDoc (xml);
 	if (ret < 0) {
 		gnumeric_io_error_save (context, "");
-		return FALSE;
 	}
-
-	return TRUE;
 }
 
 void
@@ -3316,9 +3468,12 @@ xml_init (void)
 {
 	gchar *desc = _("Gnumeric XML file format");
 
-	xml_opener = file_format_register_open (
-	             50, desc, xml_probe, gnumeric_xml_read_workbook);
-	xml_saver = file_format_register_save (
-	            "gnumeric", desc, FILE_FL_AUTO,
-	            gnumeric_xml_write_workbook);
+	xml_opener = gnum_file_opener_new (
+	             "Gnumeric_XmlIO:gnum_xml", desc,
+	             xml_probe, gnumeric_xml_read_workbook);
+	xml_saver = gnum_file_saver_new (
+	            "Gnumeric_XmlIO:gnum_xml", "gnumeric", desc,
+	            FILE_FL_AUTO, gnumeric_xml_write_workbook);
+	register_file_opener (xml_opener, 50);
+	register_file_saver_as_default (xml_saver, 50);
 }
