@@ -3,12 +3,15 @@
  *
  * Daniel Veillard <Daniel.Veillard@w3.org>
  *
- * $Id: xml-io.h,v 1.28 2001/04/28 03:47:51 jody Exp $
+ * $Id: xml-io.h,v 1.33 2001/07/10 15:45:35 lutz Exp $
  */
 
 #ifndef GNUMERIC_XML_IO_H
 #define GNUMERIC_XML_IO_H
 
+#ifdef ENABLE_BONOBO
+#include <bonobo/bonobo-stream.h>
+#endif
 #include "gnumeric.h"
 #include "file.h"
 #include "gnome-xml/tree.h"
@@ -22,9 +25,10 @@ typedef enum
     GNUM_XML_V4,	/* >= 0.57 */
     GNUM_XML_V5,	/* >= 0.58 */
     GNUM_XML_V6,	/* >= 0.62 */
-
+    GNUM_XML_V7,        /* >= 0.66 */
+    
     /* NOTE : Keep this up to date */
-    GNUM_XML_LATEST = GNUM_XML_V6
+    GNUM_XML_LATEST = GNUM_XML_V7
 } GnumericXMLVersion;
 
 typedef struct _XmlParseContext XmlParseContext;
@@ -43,6 +47,7 @@ struct _XmlParseContext {
 	xmlNodePtr parent;	/* used only for g_hash_table_foreach callbacks */
 	Sheet *sheet;		/* the associated sheet */
 	Workbook *wb;		/* the associated workbook */
+	IOContext *io_context;
 	GHashTable *style_table;/* old style styles compatibility */
 	GHashTable *expr_map;	/*
 				 * Emitted expressions with ref count > 1
@@ -55,13 +60,20 @@ struct _XmlParseContext {
 	GnumericXMLVersion    version;
 };
 
-FileOpener *gnumeric_xml_get_opener (void);
-FileSaver  *gnumeric_xml_get_saver (void);
+GnumFileOpener *gnumeric_xml_get_opener (void);
+GnumFileSaver  *gnumeric_xml_get_saver (void);
 
-gboolean gnumeric_xml_read_workbook (FileOpener const *fo, IOContext *context,
-                                     WorkbookView *wbv, const gchar *filename);
-gboolean gnumeric_xml_write_workbook (FileSaver const *fs, IOContext *context,
-                                      WorkbookView *wb_view, const gchar *filename);
+void gnumeric_xml_read_workbook (GnumFileOpener const *fo, IOContext *context,
+                                 WorkbookView *wbv, const gchar *filename);
+void gnumeric_xml_write_workbook (GnumFileSaver const *fs, IOContext *context,
+                                  WorkbookView *wbv, const gchar *filename);
+#ifdef ENABLE_BONOBO
+void gnumeric_xml_write_workbook_to_stream (GnumFileSaver const *fs, 
+		                            IOContext *context,
+					    WorkbookView *wbv,
+					    BonoboStream *stream, 
+					    CORBA_Environment *ev);
+#endif
 
 XmlParseContext *xml_parse_ctx_new      (xmlDocPtr             doc,
 					 xmlNsPtr              ns);
@@ -74,7 +86,7 @@ XmlParseContext *xml_parse_ctx_new_full (xmlDocPtr             doc,
 void             xml_parse_ctx_destroy  (XmlParseContext      *ctxt);
 
 xmlNodePtr       xml_workbook_write     (XmlParseContext      *ctx,
-					 WorkbookView         *wb_view);
+					 WorkbookView         *wbv);
 gboolean         xml_workbook_read      (IOContext            *context,
 					 WorkbookView	      *new_wb,
 					 XmlParseContext      *ctx,
